package de.mhus.lib;

import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.config.IFlatConfig;
import de.mhus.lib.lang.MObject;

public class MActivator extends MObject {

	private HashMap<String, String> mapper = new HashMap<String, String>();
	protected ClassLoader loader;

	public MActivator() {
		this(Thread.currentThread().getContextClassLoader());
	}
	
	public MActivator(ClassLoader loader) {
		this.loader=loader;
	}
	
	public MActivator(IConfig cactivator, ClassLoader loader) {
		if (loader == null) loader = Thread.currentThread().getContextClassLoader();
		this.loader=loader;
		if (cactivator != null) {
			if (cactivator instanceof IFlatConfig) {
				for (String key : cactivator.getPropertyKeys())
					addMap(key, cactivator.getExtracted(key));
			} else {
				for (IConfig entry : cactivator.getConfigBundle("map"))
					addMap(entry.getExtracted("name",""), entry.getExtracted("class",""));
			}
		}
	}

	public void addMap(String name, String clazz) {
		if (name == null || clazz == null) return;
		log().t("add map",name,clazz);
		mapper.put(name, clazz);
	}
	
	public Object createObject(String name) throws Exception {
		return createObject(name, null, null);
	}
	
	public String mapName(String name) {
		if (name == null) return name;
		if (mapper.containsKey(name))
			name = mapper.get(name);
		return name;
	}
	
	public Object createObject(String name, Class<?>[] classes, Object[] objects ) throws Exception {
		
		name = mapName(name);
		
		if (classes == null || objects == null)
			return loader.loadClass(name).newInstance();
		return loader.loadClass(name).getConstructor(classes).newInstance(objects);
	}
	
	public InputStream getResource(String name) throws Exception {
		name = mapName(name);
		return loader.getResourceAsStream(name);
	}

	public Object getObject(String name) throws Exception {
		return createObject(name);
	}

	public Class<?> getClazz(String name) throws Exception {
		name = mapName(name);
		return loader.loadClass(name);
	}

	public URL getURL(String path) {
		return loader.getResource(path);
	}
	
}
