package de.mhus.lib;

import java.io.File;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.jmx.JmxManaged;
import de.mhus.lib.jmx.MJmx;
import de.mhus.lib.persistence.RoundRobin;

@JmxManaged(descrition = "Simple Counter")
public class MCount extends MJmx {

	private long cnt;
	private String name;
	private long startTime = 0;
	private long lastTime = 0;
	private RoundRobin rrd;
	private boolean isClosed;
	
	public MCount() {
		cnt = 0;
		name = "Counter " + MSingleton.instance().nextUniqueId();
		persistence();
	}
	
	public MCount(String name) {
		cnt = 0;
		this.name = name;
		persistence();
	}
	
	private void persistence() {
		try {
			if (MSingleton.instance().isPersistence()) {
				File f = MSingleton.instance().getPersistenceManager().sessionScope().getPersistenceFile(getJmxName() + "_" + name + ".rrd");
				rrd = new RoundRobin(MTimeInterval.HOUR_IN_MILLISECOUNDS , f);
			}
		} catch (Throwable t) {
			log().t(t);
		}
	}

	@JmxManaged(descrition="Reset the counter statistic")
	public void reset() {
		isClosed = false;
		cnt = 0;
		startTime = 0;
		lastTime = 0;
	}
	
	public void inc() {
		if (isClosed) return;
		cnt++;
		lastTime = System.currentTimeMillis();
		if (startTime == 0) startTime = lastTime;
		if (rrd != null) try {rrd.add(cnt);} catch(Throwable t) {}
	}

	@JmxManaged(descrition = "Amount of counts")
	public long getValue() {
		return cnt;
	}
	
	public double getHitsPerSecond() {
		if (startTime == 0 || lastTime == 0 || cnt == 0) return 0;
		return (double)cnt / (double)((lastTime - startTime)/1000);
	}
	
	@JmxManaged(descrition = "Name of this value")
	public String getName() {
		return name;
	}
	
	public long getFirstHitTime() {
		return startTime;
	}
	
	public long getLastHitTime() {
		return lastTime;
	}
	
	@JmxManaged(descrition = "Readable status of the counter")
	public String getStatusAsString() {
		if (startTime == 0 || lastTime == 0 || cnt == 0) return "unused";
		return MCast.toIsoDateTime(getFirstHitTime()) + " - " + MCast.toIsoDateTime(getLastHitTime()) + "," + getHitsPerSecond() + " hits/sec," + cnt;
	}
	
	public void close() {
		if (isClosed) return;
		isClosed = true;
		log().i("close",name,cnt,getHitsPerSecond());
		try {
			if (MSingleton.instance().isPersistence()) {
				IConfig persistence = MSingleton.instance().getPersistenceManager().sessionScope().getPersistence("de.mhus.lib");
				long uid = MSingleton.instance().nextUniqueId();
				persistence.setString(getJmxName() + "_" + name + "_" + uid, getStatusAsString());
				if (rrd != null) {
					persistence.setString(getJmxName() + "_" + name + "_rrd_" + uid, rrd.dump());
				}
				persistence.save();
			}
		} catch (Throwable t) {
			log().t(t);
		}
	}
	
	protected void finalize() {
		close();
	}
	
	public String toString() {
		return name + "=" + getStatusAsString() + "@MCount";
	}
}
