package de.mhus.lib.aao;

import de.mhus.lib.cao.CaoApplication;
import de.mhus.lib.cao.CaoException;
import de.mhus.lib.config.IConfig;

/**
 * This is the Abstraction Access Object and is a next level (level 2) abstraction for
 * Content Access Object (cao). The scope is to define a application on top of
 * the cao as data layer (level 1). It should be easy to switch the underlying cao
 * implementation but not to change the level 2 interface.
 * 
 * Implement a AaoFactory to create your own objects which must extend the AaoElement.
 * 
 * @author mikehummel
 *
 */
public abstract class AaoApplication extends CaoApplication {

	protected CaoApplication slave;
	private AaoFactory factory;

	protected AaoApplication(CaoApplication slave, AaoFactory factory, IConfig config)
			throws CaoException {
		super(slave.getConnection(), config);
		this.slave = slave;
		this.factory = factory;
	}

	public AaoFactory getFactory() {
		return factory;
	}
}
