package de.mhus.lib.aao;

import de.mhus.lib.cao.CaoElement;
import de.mhus.lib.cao.CaoException;
import de.mhus.lib.cao.CaoList;
import de.mhus.lib.cao.CaoListIterator;
import de.mhus.lib.cao.CaoMetadata;

public class AaoList extends CaoList {

	private CaoList slaveList;
	private AaoFactory factory;

	public AaoList(AaoElement parent, CaoList slaveList) {
		super(parent,null);
		factory = ((AaoApplication)parent.getApplication()).getFactory();
		this.slaveList = slaveList;
	}

	@Override
	public CaoMetadata getMetadata() {
		return slaveList.getMetadata();
	}

	@Override
	public int size() {
		return slaveList.size();
	}

	@Override
	public CaoListIterator getElements() throws CaoException {
		return new AaoIterator(slaveList.getElements());
	}

	public AaoFactory getFactory() {
		return factory;
	}
	
	
	private class AaoIterator extends CaoListIterator {

		private CaoListIterator slave;

		public AaoIterator(CaoListIterator slave) {
			super(AaoList.this.getConnection());
			this.slave = slave;
		}

		@Override
		protected CaoElement nextElement() throws CaoException {
			return getFactory().create(getParent(),slave.next());
		}

		@Override
		protected boolean hasNextElement() throws CaoException {
			return slave.hasNext();
		}

		@Override
		public void reset() throws CaoException {
			slave.reset();
		}
		
	}
}
