package de.mhus.lib.adb;

import de.mhus.lib.MException;
import de.mhus.lib.lang.MObject;
import de.mhus.lib.sql.DbConnection;

/**
 * A comfortable abstract object base to make the life of the developer fresh and
 * handy. Use this, if possible as base of a database object. You will love it.
 * 
 * @author mikehummel
 *
 */
public class DbComfortableObject extends MObject implements DbObject {

	protected DbManager manager;
	protected String registryName;
	protected DbConnection con;

	public boolean isAdbManaged() {
		return manager != null;
	}
	/**
	 * Save changes.
	 * 
	 * @throws MException
	 */
	public void save() throws MException {
		manager.saveObject(con, registryName,this);
	}
	
	/**
	 * Save changed is not jet managed it will create the object
	 * 
	 * @param manager
	 * @throws MException 
	 */
	public void save(DbManager manager) throws MException {
		if (isAdbManaged())
			save();
		else
			create(manager);
	}
		
	/**
	 * Reload from database.
	 * 
	 * @throws MException
	 */
	public void reload() throws MException {
		manager.reloadObject(con, registryName, this);
	}
	
	/**
	 * Remove in database.
	 * 
	 * @throws MException
	 */
	public void remove() throws MException {
		if (isAdbManaged()) 
			manager.removeObject(con, registryName,this);
	}
	
	/**
	 * Create this new object in the database.
	 * 
	 * @param manager
	 * @throws MException
	 */
	public void create(DbManager manager) throws MException {
		manager.createObject(con, this);
	}
	
	/**
	 * Overwrite to get the hook.
	 */
	@Override
	public void doPreCreate(DbConnection con) {
	}

	/**
	 * Overwrite to get the hook.
	 */
	@Override
	public void doPreSave(DbConnection con) {
	}

	/**
	 * Overwrite to get the hook.But remember, it could be called more times. It's only
	 * to store the data.
	 */
	@Override
	public void doInit(DbManager manager, String registryName) {
		this.manager = manager;
		this.registryName = registryName;
	}

	/**
	 * Overwrite to get the hook.
	 */
	@Override
	public void doPreRemove(DbConnection con) {
	}

	/**
	 * Overwrite to get the hook.
	 */
	@Override
	public void doPostLoad(DbConnection con) {
	}

	/**
	 * Overwrite to get the hook.
	 */
	@Override
	public void doPostRemove(DbConnection con) {
	}

	public DbManager getDbManager() {
		return manager;
	}

}
