package de.mhus.lib.adb.model;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import de.mhus.lib.MCast;
import de.mhus.lib.MException;
import de.mhus.lib.adb.DbDynamic;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.lang.MObject;
import de.mhus.lib.sql.DbResult;

public abstract class Field extends MObject {

	protected boolean isPrimary;
	protected String name;
	protected String nameOrg;
	protected String createName;
	protected Method setter;
	protected Method getter;
	protected Table table;
	protected boolean nullable = true;
	protected String defValue = null;
	protected int size = 200;
	protected Class<?> ret;
	protected String retDbType;
	protected String methodName;
	protected boolean autoId;
	protected IConfig attr;
	protected DbDynamic.Field dynamicField;

	public abstract void prepareCreate(Object obj) throws Exception;
	public abstract boolean isPersistent();
	public abstract Object getFromTarget(Object obj) throws Exception;
	public abstract void setToTarget(DbResult res, Object obj) throws Exception;
	public abstract void fillNameMapping(HashMap<String, Object> nameMapping);

	public void set(Object obj, Object value) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException, MException {
		
		if (ret.isEnum()) {
			int index = -1;
			if (value == null)
				index= MCast.toint(defValue, -1);
			else
			if (value instanceof Number)
				index = ((Number)value).intValue();

			Object[] values=ret.getEnumConstants();
			if (index < 0 || index >= values.length) throw new MException("index not found in enum",ret.getName());
			
			value = values[index];
			
		}
		
		for (Feature f : table.getFeatures())
			value = f.set(obj, this, value);

		if (dynamicField != null && obj instanceof DbDynamic)
			((DbDynamic)obj).setValue(dynamicField,value);
		else
			setter.invoke(obj, value);
	}
	
	public Object get(Object obj) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		Object val = null;
		if (dynamicField != null && obj instanceof DbDynamic)
			val = ((DbDynamic)obj).getValue(dynamicField);
		else
			val = getter.invoke(obj);
		
		for (Feature f : table.getFeatures())
			val = f.get(obj, this, val);
				
		return val;
	}
	
	public IConfig getAttributes() {
		return attr;
	}
	public int getSize() {
		return size;
	}


}
