package de.mhus.lib.adb.model;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import de.mhus.lib.MCast;
import de.mhus.lib.MDate;
import de.mhus.lib.MException;
import de.mhus.lib.adb.DbDynamic;
import de.mhus.lib.adb.DbManager;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.lang.Raw;
import de.mhus.lib.sql.DbResult;
import de.mhus.lib.sql.Dialect;

public class FieldPersistent extends Field {

	private DbManager manager;

	public FieldPersistent(DbManager manager, Table table, String name, boolean isPrimary, Method setter,
			Method getter, Class<?> ret, IConfig attr2,DbDynamic.Field dynamicField) {
		this.manager = manager;
		this.table = table;
		this.nameOrg = name;
		this.name = manager.getPool().getDialect().normalizeColumnName(nameOrg.toLowerCase());
		this.createName = name.toLowerCase();
		this.methodName = name;
		this.isPrimary = isPrimary;
		this.setter = setter;
		this.getter = getter;
		this.ret = ret;
		this.attr = attr2;
		this.dynamicField = dynamicField;
		init();
	}

//	public FieldPersistent(DbManager manager, Table table, DbDynamic.Field f) {
//		this.manager = manager;
//		this.table = table;
//		methodName = f.getName();
//		this.nameOrg = methodName;
//		this.name = manager.getPool().getDialect().normalizeColumnName(methodName.toLowerCase());
//		this.createName = methodName.toLowerCase();
//		this.isPrimary = f.isPrimaryKey();
//		this.ret = f.getReturnType();
//		this.attr = f.getAttributes();
//		this.dynamicField = f;
//		init();
//	}

	protected void init() {
		this.retDbType = attr.getExtracted("type", table.getDbRetType(ret) ).toUpperCase();
		this.autoId = attr.getBoolean("auto_id",false);
		size = attr.getInt("size", size );
		defValue = attr.getString("default", null);
		nullable = attr.getBoolean("nullable", true);
		
		if (isPrimary) nullable = false;
	}
	
	public void fillNameMapping(HashMap<String, Object> nameMapping) {
		nameMapping.put("db." + table.getName() + "." + methodName, new Raw(name));
		if (ret.isEnum()) {
			int cnt = 0;
			for ( Object c : ret.getEnumConstants()) {
				nameMapping.put("db." + table.name + "." + methodName + "." + c, cnt);
				cnt++;
			}
		}
	}

	public void prepareCreate(Object obj) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException, MException {
		if (autoId) {
			if (ret == UUID.class) {
				UUID uuid = UUID.randomUUID();
				set(obj,uuid);
			} else
			if (ret == String.class) {
				UUID uuid = UUID.randomUUID();
				set(obj, uuid.toString() );
			} else
			if (ret == long.class) {
				long id = manager.getSchema().getUniqueId(table,this,obj,name,manager);
				set(obj, id );
			}
			else
				log().i("can't set uuid to object",name);
		}
	}

	public Object getFromTarget(Object obj) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException, IOException {
		Object out = get(obj);
		if (retDbType.equals(Dialect.TYPE.BLOB.name())) {
			ByteArrayOutputStream os = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(os);
			oos.writeObject(out);
			ByteArrayInputStream is = new ByteArrayInputStream(os.toByteArray());
			return is;
		}
		return out;
	}

	public void setToTarget(DbResult res, Object obj) throws Exception {
		
		if (retDbType.equals(Dialect.TYPE.INT.name()))
			set(obj, res.getInt(name) );
		else
		if (retDbType.equals(Dialect.TYPE.LONG.name()))
			set(obj, res.getLong(name) );
		else
		if (retDbType.equals(Dialect.TYPE.BOOL.name()))
			set(obj, res.getBoolean(name) );
		else
		if (retDbType.equals(Dialect.TYPE.DATETIME.name())) {
			if (ret == Date.class)
				set(obj, res.getDate(name) );
			else
				set(obj, new MDate( res.getDate(name) ).toCalendar() );
		} else
		if (retDbType.equals(Dialect.TYPE.DOUBLE.name()))
			set(obj, res.getDouble(name) );
		else
		if (retDbType.equals(Dialect.TYPE.FLOAT.name()))
			set(obj, res.getFloat(name) );
		else
		if (retDbType.equals(Dialect.TYPE.STRING.name()))
			set(obj, res.getString(name) );
		else
		if (retDbType.equals(Dialect.TYPE.UUID.name())) {
			String o = res.getString(name);
			if (o == null)
				set(obj, (UUID)null );
			else
				try {
					set(obj, UUID.fromString(o) );
				} catch (Throwable t) {
					log().d("uuid",name,o,t);
					set(obj, (UUID)null );
				}
		} else
		if (retDbType.equals(Dialect.TYPE.BLOB.name())) {
			InputStream st = res.getBinaryStream(name);
			if (st != null) {
				ObjectInputStream ois = new ObjectInputStream(st);
				Object o = ois.readObject();
				set(obj, o );
			} else
				set(obj,null);
		} else
			log().d("can't set to target ",name,retDbType );
	}

	@Override
	public boolean isPersistent() {
		return true;
	}
	
}
