package de.mhus.lib.cao;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.logging.Log;

public abstract class CaoApplicationFactory {

	private static Log log = Log.getLog(CaoApplicationFactory.class);
	
	protected IConfig config;
	private String name;

	public abstract CaoForm createConfiguration();

	/**
	 * Return the created application. The method will search for the application config. If the config is
	 * found, it calls the create(con,config) to trigger creation.
	 * 
	 * Searching the config is done by loading the app config or (if is null) the singleton config. The path is
	 * /cao/<driver service name>/<application name>
	 * 
	 * @param con
	 * @return
	 */
	public CaoApplication create(CaoConnection con) {
		IConfig c = config;
		if ( c == null) {
			c = con.getDriver().getConfig();
		}
		if (c != null) {
			// IConfig driverConfig = c.getConfig(con.getDriver().getName());
			// if (driverConfig != null) {
				IConfig appConfig = c.getConfig(getName());
				if (appConfig != null) {
					try {
						return create(con, appConfig);
					} catch (CaoException e) {
						log.e(con.getDriver().getName(),getName(),e);
					}
				}
//			}
		}
		log.i("application not created",con.getDriver().getName(),getName());
		return null;
	}

	/**
	 * Overwrite this to create the application.
	 * 
	 * @param con
	 * @param config2
	 * @return
	 */
	protected abstract CaoApplication create(CaoConnection con, IConfig config2) throws CaoException;

	public void setConfig(IConfig config) {
		this.config = config;
	}

	void setName(String name) {
		this.name = name;
	}
	
	public String getName() {
		return name;
	}
	
}
