package de.mhus.lib.cao;

import java.util.HashMap;

import de.mhus.lib.MActivator;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.logging.Log;

public class CaoFactory {

	private static Log log = Log.getLog(CaoFactory.class);
	
	private HashMap<String, Object[]> registry = new HashMap<String, Object[]>();
	private HashMap<String, CaoDriver> drivers  = new HashMap<String, CaoDriver>();
	
	//private static LinkedList<CaoActionProvider> actionProviders = new LinkedList<CaoActionProvider>();
	private static CaoActionForwarder actionProviders = new CaoActionForwarder();
	
	public CaoDriver getDriver(String service) throws CaoException {
		
		synchronized (drivers) {
			CaoDriver driver = drivers.get(service);
			
			if (driver==null) {
				Object[] definition = registry.get(service);
				try {
					log.t("load driver",definition);
					driver = (CaoDriver)((MActivator)definition[2]).createObject(CaoDriver.ACTIVATOR_SERVICE);
					driver.initialize(service, null, (MActivator)definition[2]);
				} catch (Exception e) {
					// e.printStackTrace();
					throw new CaoException("Init of driver failed: " + service, e);
				}
//				if (driver==null)
//					throw new CaoException("Init of driver failed: " + service);
				drivers.put(service, driver);
			}
			return driver;
		}
		
	}
	
	public CaoConnection createConnection(String service, CaoForm form) throws CaoException {
		return getDriver(service).createConnection(form);		
	}
	
	public CaoConnection createConnection(String service,String url, IConfig config) throws CaoException {
		return getDriver(service).createConnection(url,config);
	}
	
	public CaoForm createConfiguration(String service) throws CaoException {
		return getDriver(service).createConfiguration();
	}
	
	public void registerDriver(String id, String title, MActivator activator) {
		synchronized (drivers) {
			log.t("register driver",id,title);
			registry.put(id, new Object[] {id,title,activator});
		}
	}
	
	public void registerActionProvider(IActionProvider actionProvider) {
		synchronized (actionProviders) {
			log.t("register action provider",actionProvider);
			if (actionProviders.contains(actionProvider)) return;
			actionProviders.add(actionProvider);			
		}
	}
	
	public void fillWithActions(CaoElement element, CaoList targetList, CaoActionList actionList,Object...initConfig) {
		synchronized (actionProviders) {
			// ask the driver for actions. The driver will ask the application and the connection ...
			if (element != null) {
				element.getDriver().fillWithActions(element, targetList, actionList, initConfig);
			}
			targetList.getDriver().fillWithActions(element,targetList, actionList, initConfig);
			// Ask custom - general - providers
			actionProviders.fillWithActions(element,targetList, actionList, initConfig);
		}
	}

	public MActivator getActivator(String service) throws CaoException {
		Object[] definition = registry.get(service);
		if (definition== null)
			throw new CaoException("Service not found: " + service);
		return (MActivator) definition[2];
	}
	
}
