package de.mhus.lib.cao;

import java.util.Iterator;

public abstract class CaoListIterator implements Iterable<CaoElement>, Iterator<CaoElement> {

	private CaoConnection connection;
	private int backCounter = -1;
	
	protected CaoListIterator(CaoConnection connection) {
		this.connection = connection;
	}

	public final CaoConnection getConnection() {
		return connection;
	}
	
	/**
	 * Set a maximum counter for the next inetration. In this way a real iteration can be split into
	 * parts. After reset the hole iterator is available.
	 * @param count
	 */
	public void setNextIteratorCount(int count) {
		backCounter=count;
	}

	public void resetNextIteratorCount() {
		backCounter = -1;
	}

	protected abstract CaoElement nextElement() throws CaoException;
	
	protected abstract boolean hasNextElement() throws CaoException;
	
	public CaoElement next() {
		
		if ( backCounter == 0 ) return null;
		if ( backCounter >  0 ) backCounter--;
		
		try {
			return nextElement();
		} catch (CaoException e) {
			return null;
		}
	}
	
	public boolean hasNext() {
		if ( backCounter == 0 ) return false;		
		try {
			return hasNextElement();
		} catch (CaoException e) {
			return false;
		}
	}
	
	public void flush(int nr) throws CaoException {
		for (int i = 0; i < nr; i++)
			if (hasNextElement()) nextElement();
	}
	
	@Override
	public Iterator<CaoElement> iterator() {
		return this;
	}
	
	@Override
	public final void remove() {
	}

	/**
	 * Reset the iterator to the first element.
	 */
	public abstract void reset() throws CaoException;


}
