package de.mhus.lib.cao;

import de.mhus.lib.MEventHandler;

public class CaoObserver implements ICaoChangeListener {

	private CaoConnection connection;
	private static de.mhus.lib.logging.Log log = de.mhus.lib.logging.Log.getLog(CaoObserver.class);

	private ChangedHandler handler = new ChangedHandler();
	private ICaoChangeListener listener = null;
	
	public CaoObserver(CaoConnection connection) {
		this.connection=connection;
	}
	
	public CaoConnection getConnection() {
		return connection;
	}

	public void setListener(ICaoChangeListener listener) {
		this.listener=listener;
	}
	
	@Override
	public void elementCreated(CaoConnection con, String id) {
		handler.fireElementCreated(con,id);
	}

	@Override
	public void elementDeleted(CaoConnection con, String id) {
		handler.fireElementDeleted(con,id);
	}

	@Override
	public void elementUpdated(CaoConnection con,  String id) {
		handler.fireElementUpdated(con,id);
	}
	
	@Override
	public void elementLink(CaoConnection con, String parentId, String id) {
		handler.fireElementLink(con,parentId,id);
	}
	
	@Override
	public void elementUnlink(CaoConnection con, String parentId, String id) {
		handler.fireElementUnlink(con,parentId,id);
	}
	
	@Override
	public void elementStructurChanged(CaoConnection con, String id) {
		handler.fireElementStructurChanged(con,id);
	}

	public void add(CaoElement elementToListen) {
		if (elementToListen==null) return;
		handler.register(elementToListen);
	}
	
	public void remove(CaoElement elementToListen) {
		if (elementToListen==null) return;
		handler.unregister(elementToListen);
	}
	
	private class ChangedHandler extends MEventHandler<CaoElement> {
		ChangedHandler() {
			super(true);
		}
		
		public void fireElementStructurChanged(CaoConnection con, String id) {
			for (CaoElement item : getListeners()) {
				try {
					if (id == null || id.equals(item.getId())) {
						if (listener!=null && listener.isValid()) listener.elementStructurChanged(con,id);
					}
				} catch (Throwable t) {
					log.debug(t);
				}
			}
		}

		public void fireElementLink(CaoConnection con,String parentId, String id) {
			for (CaoElement item : getListeners()) {
				try {
					if (parentId.equals(item.getId())) {
						if (listener!=null && listener.isValid()) listener.elementLink(con,parentId,id);
					}
				} catch (Throwable t) {
					log.debug(t);
				}
			}			
		}

		public void fireElementUnlink(CaoConnection con,String parentId, String id) {
			for (CaoElement item : getListeners()) {
				try {
					if (parentId.equals(item.getId())) {
						if (listener!=null && listener.isValid()) listener.elementUnlink(con,parentId,id);
					}
				} catch (Throwable t) {
					log.debug(t);
				}
			}			
		}
		
		public void fireElementUpdated(CaoConnection con, String id) {
			for (CaoElement item : getListeners()) {
				try {
					if (id.equals(item.getId())) {
						if (listener!=null && listener.isValid()) listener.elementUpdated(con,id);
					}
				} catch (Throwable t) {
					log.debug(t);
				}
			}
		}

		public void fireElementCreated(CaoConnection con,String id) {
			for (CaoElement item : getListeners()) {
				try {
					if (id.equals(item.getId())) {
						if (listener!=null && listener.isValid()) listener.elementCreated(con, id);
					}
				} catch (Throwable t) {
					log.debug(t);
				}
			}
		}

		public void fireElementDeleted(CaoConnection con,String id) {
			for (CaoElement item : getListeners()) {
				try {
					if (id.equals(item.getId())) {
						if (listener!=null && listener.isValid()) listener.elementDeleted(con, id);
					}					
				} catch (Throwable t) {
					log.debug(t);
				}
			}
		}

		public void fireDisconnected(CaoConnection con) {
			if (listener!=null && listener.isValid()) listener.disconnected(con);
		}
	}

	@Override
	public boolean isValid() {
		return true;
	}

	@Override
	public void disconnected(CaoConnection con) {
		handler.fireDisconnected(con);
	}

	
}
