package de.mhus.lib.cao;

public interface ICaoChangeListener {

	/**
	 * Returns true 
	 * @return
	 */
	public boolean isValid();
	
	/**
	 * Fired if the data of an element changed. Fired after the change.
	 * @param con
	 * @param id
	 */
	public void elementUpdated(CaoConnection con, String id);
	
	/**
	 * Fired if a element is created.
	 * @param con
	 * @param id
	 */
	public void elementCreated(CaoConnection con, String id);
	
	public void elementDeleted(CaoConnection con, String id);
	
	/**
	 * Linked a node to a parent node.
	 * 
	 * @param con
	 * @param parentId
	 * @param id
	 */
	public void elementLink(CaoConnection con, String parentId, String id);
	
	/**
	 * Unlinked a element from a parent node.
	 * 
	 * @param con
	 * @param parentId
	 * @param id
	 */
	public void elementUnlink(CaoConnection con, String parentId, String id);

	/**
	 * Fired if the internal structure of an element changed, e.g. if a attributed is added.
	 * @param con
	 * @param id Id of the element or null if all elements should be updated.
	 */
	public void elementStructurChanged(CaoConnection con, String id);

	/**
	 * Fired after the connection is disconnected.
	 * 
	 * @param caoConnection
	 */
	public void disconnected(CaoConnection caoConnection);
	
}
