package de.mhus.lib.cao.util;

import java.util.Map.Entry;

import de.mhus.lib.MException;
import de.mhus.lib.MString;
import de.mhus.lib.cao.CaoMetaDefinition;
import de.mhus.lib.cao.CaoMetadata;
import de.mhus.lib.config.IConfig;

public class CaoExport {

	
	/**
	 * Fill the config with table definitions. To have a full db definition use
	 * IConfig cDef = config.createConfig("definitions");
	 * and fill it with the tables.
	 * 
	 * @param bundle
	 * @param config
	 * @throws MException
	 */
	public static void fillTables(MetadataBundle bundle, IConfig config) throws MException {
		for (Entry<String, CaoMetadata> meta : bundle.getBundle().entrySet()) {
			IConfig cTab = config.createConfig("table");
			cTab.setProperty("name", meta.getKey());
			fillColumns(meta.getValue(),cTab);
		}
	}

	/**
	 * Fill a config with column definitions of a table.
	 * 
	 * @param meta
	 * @param config
	 * @throws MException
	 */
	public static void fillColumns(CaoMetadata meta, IConfig config) throws MException {
		for (CaoMetaDefinition m : meta) {
			IConfig cM = config.createConfig("column");

			cM.setProperty("name", m.getName());
			cM.setProperty("type", m.getType().toString() );
			if (m.getNls() != null) 
				cM.setProperty("nls", m.getNls());
			if (m.getSize() > 0)
				cM.setProperty("size", String.valueOf(m.getSize()) );
			if(m.getCategories() != null)
				cM.setProperty("categories", MString.join(m.getCategories(),',') );
		}
	}
}
