package de.mhus.lib.cao.util;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

import de.mhus.lib.cao.CaoAccess;
import de.mhus.lib.cao.CaoApplication;
import de.mhus.lib.cao.CaoElement;
import de.mhus.lib.cao.CaoException;
import de.mhus.lib.cao.CaoList;
import de.mhus.lib.cao.CaoMetadata;
import de.mhus.lib.cao.CaoNotFoundException;
import de.mhus.lib.cao.CaoNotSupportedException;
import de.mhus.lib.cao.CaoWritableElement;
import de.mhus.lib.cao.ObjectNotFoundException;
import de.mhus.lib.util.ArraySet;


public class EmptyElement extends CaoElement {

	private String name;
	private CaoList children;
	private CaoMetadata meta;

	public EmptyElement(CaoApplication app, String name) throws CaoException {
		this(app,name,(List<CaoElement>)null);
	}
	
	public EmptyElement(CaoApplication app, String name,List<CaoElement> children) throws CaoException {
		super(app);
		this.name = name;
		this.meta = new CaoMetadata(getDriver());
		if (children != null)
			this.children = new LinkedCaoList(this, children);
	}
	
	public EmptyElement(CaoApplication app, String name,CaoList children) throws CaoException {
		super(app);
		this.name = name;
		this.meta = new CaoMetadata(getDriver());
		if (children != null)
			this.children = children;
	}
	
	public EmptyElement(CaoElement parent, String name) throws CaoException {
		this(parent,name,null);
	}
	
	public EmptyElement(CaoElement parent, String name,List<CaoElement> children) throws CaoException {
		super(parent);
		this.name = name;
		this.meta = new CaoMetadata(getDriver());
		if (children != null)
			this.children = new LinkedCaoList(this, children);
	}

	@Override
	public CaoList getChildren(CaoAccess CaoAccess) throws CaoException {
		if (children != null) return children;
		throw new ObjectNotFoundException();
	}

	@Override
	public String getId() throws CaoException {
		throw new ObjectNotFoundException();
	}

	@Override
	public CaoList getList(String name, CaoAccess CaoAccess, String... attributes)
			throws CaoException {
		throw new ObjectNotFoundException(name);
	}

	@Override
	public CaoMetadata getMetadata() {
		return meta;
	}

	@Override
	public String getName() throws CaoException {
		return name;
	}

	@Override
	public Object getObject(String name, String... attributes)
			throws CaoException {
		throw new CaoNotFoundException(this,name);
	}

	@Override
	public String getString(String name) throws CaoException {
		throw new CaoNotFoundException(this,name);
	}

	@Override
	public CaoWritableElement getWritableNode() throws CaoException {
		throw new CaoNotSupportedException(this);
	}

	@Override
	public boolean isLocked(boolean owner) throws CaoException {
		return owner;
	}

	@Override
	public boolean isNode() {
		return false;
	}

	@Override
	public boolean isValid() {
		return true;
	}

	@Override
	public boolean isWritable() {
		return false;
	}

	@Override
	public boolean lock(int timeout) throws CaoException {
		return false;
	}

	@Override
	public void reload() throws CaoException {
	}

	@Override
	public boolean unlock() throws CaoException {
		return false;
	}

	@Override
	public boolean isEditable() {
		return false;
	}

	@Override
	public Set<String> keys() {
		return new ArraySet<String>(new String[0]);
	}

}
