package de.mhus.lib.cao.util;

import java.util.Iterator;
import java.util.LinkedList;

import de.mhus.lib.cao.CaoElement;
import de.mhus.lib.cao.CaoException;
import de.mhus.lib.cao.CaoList;
import de.mhus.lib.cao.CaoListIterator;
import de.mhus.lib.cao.CaoMetadata;


public class MutableCaoList extends CaoList {

	private LinkedList<CaoElement> data;
	private CombinedMetadata meta;

	public MutableCaoList(CaoElement parent, LinkedList<CaoElement> data) throws CaoException {
		super(parent, null);
		this.data = data;
		refreshMetadata();
	}

	public LinkedList<CaoElement> getList() {
		return data;
	}
	
	public void refreshMetadata() {
		meta = new CombinedMetadata(getDriver());
		for (CaoElement ele : data) {
			meta.addMetaDefinitions(ele.getMetadata());
		}
	}
	
	@Override
	public CaoMetadata getMetadata() {
		return meta;
	}

	@Override
	public int size() {
		return data.size();
	}


	private class ListIterator extends CaoListIterator {

		private Iterator<CaoElement> iterator;
		
		protected ListIterator() {
			super(MutableCaoList.this.getConnection());
			reset();
		}


		@Override
		public boolean hasNextElement() {
			return iterator.hasNext();
		}

		@Override
		public CaoElement nextElement() {
			return iterator.next();
		}

		@Override
		public void reset() {
			iterator = data.iterator();
		}
		
	}


	@Override
	public CaoListIterator getElements() {
		return new ListIterator();
	}

	@Override
	public CaoElement getParent() {
		return null;
	}
	
}
