package de.mhus.lib.cao.util;

import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import de.mhus.lib.MException;
import de.mhus.lib.MThread;
import de.mhus.lib.cao.CaoAccess;
import de.mhus.lib.cao.CaoApplication;
import de.mhus.lib.cao.CaoElement;
import de.mhus.lib.cao.CaoException;
import de.mhus.lib.cao.CaoInvalidException;
import de.mhus.lib.cao.CaoList;
import de.mhus.lib.cao.CaoMetaDefinition;
import de.mhus.lib.cao.CaoMetadata;
import de.mhus.lib.cao.CaoNotFoundException;
import de.mhus.lib.cao.CaoWritableElement;
import de.mhus.lib.util.NullableMap;

public class MutableElement extends CaoElement{

	
	private boolean isNode = false;
	private String id = "?";
	private String name = "?";
	private MutableMetadata meta;
	protected NullableMap<String,Object> data = new NullableMap<String, Object>();
	private boolean writable = true;
	private boolean locked = false;
	private boolean valid = true;
	private CaoList children;

	public MutableElement(CaoApplication app) throws CaoException {
		super(app);
		meta = new MutableMetadata(getDriver());
	}

	public MutableElement(CaoElement parent) throws CaoException {
		super(parent);
		meta = new MutableMetadata(getDriver());
	}

	@Override
	public CaoList getChildren(CaoAccess access) throws CaoException {
		return children;
	}

	public void setChildren(CaoList in) {
		children = in;
	}
	
	@Override
	public boolean isNode() {
		return isNode;
	}

	@Override
	public String getId() throws CaoException {
		return id;
	}

	@Override
	public String getName() throws CaoException {
		return name;
	}

	@Override
	public CaoMetadata getMetadata() {
		return meta;
	}
	
	public void setMetadata(MutableMetadata meta) {
		this.meta = meta;
	}
	
	public List<CaoMetaDefinition> getMetaDefinitions() {
		return meta.getMap();
	}
	
	@Override
	public String getString(String name) throws CaoException {
		if (!data.containsKey(name)) throw new CaoNotFoundException(getName(),name);
		Object out = data.get(name);
		if (out == null) return null;
		return out.toString();
	}

	@SuppressWarnings("unchecked")
	@Override
	public CaoList getList(String name, CaoAccess access, String... attributes)
			throws CaoException {
		Object out = data.get(name);
		if (out == null || ! (out instanceof List) ) throw new CaoNotFoundException(getName(),name);
		
		return new LinkedCaoList(this, (List<CaoElement>) out);
	}

	@Override
	public Object getObject(String name, String... attributes)
			throws CaoException {
		if (!data.containsKey(name)) throw new CaoNotFoundException(getName(),name);
		Object out = data.get(name);
		return out;
	}

	@Override
	public boolean isWritable() {
		return writable;
	}

	@Override
	public CaoWritableElement getWritableNode() throws CaoException {
		if (!isWritable()) throw new CaoInvalidException("Element is not writable");
		return new WritableElement(this) {
			
			@Override
			public boolean isValid() {
				return getParent().isValid();
			}
			
			@Override
			public boolean isWritable(String name) {
				return getParent().isWritable();
			}
			
			@Override
			public void save() throws CaoException {
				for (Entry<String, Object> entry : data.entrySet()) {
					MutableElement.this.setObject(entry.getKey(),entry.getValue());
				}
				MutableElement.this.save();
			}

			@Override
			public Set<String> keys() {
				return data.keySet();
			}
		};
	}

	protected void save() throws CaoException {
	}

	public void setObject(String key, Object value) throws CaoException {
		data.put(key, value);
	}

	@Override
	public void reload() throws CaoException {
		
	}

	@Override
	public boolean lock(int timeout) throws CaoException {
		if (timeout < 0) throw new CaoInvalidException("timeout less zero");
		synchronized (this) {
			while (locked) {
				MThread.sleep(100);
				if (timeout !=0) {
					timeout-=100;
					if (timeout <=0) return false;
				}
			}
			locked = true;
		}
		return true;
	}

	@Override
	public boolean unlock() throws CaoException {
		boolean out = true;
		synchronized (this) {
			out = locked;
			locked = false;
		}
		return out;
	}

	@Override
	public boolean isLocked(boolean owner) throws CaoException {
		return locked;
	}

	@Override
	public boolean isValid() {
		return valid;
	}

	public void setNode(boolean isNode) {
		this.isNode = isNode;
	}

	public void setId(String id) {
		this.id = id;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setWritable(boolean writable) {
		this.writable = writable;
	}

	public void setValid(boolean valid) {
		this.valid = valid;
	}

	@Override
	public void setProperty(String key, Object value) throws MException {
		setObject(key, value);
	}
	
	@Override
	public void removeProperty(String key) throws MException {
		data.remove(key);
	}

	@Override
	public Set<String> keys() {
		return data.keySet();
	}


}
