package de.mhus.lib.cao.util;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import de.mhus.lib.MCast;
import de.mhus.lib.MDate;
import de.mhus.lib.cao.CaoAccess;
import de.mhus.lib.cao.CaoElement;
import de.mhus.lib.cao.CaoException;
import de.mhus.lib.cao.CaoList;
import de.mhus.lib.cao.CaoMetaDefinition;
import de.mhus.lib.cao.CaoMetaDefinition.TYPE;
import de.mhus.lib.cao.CaoMetadata;
import de.mhus.lib.cao.CaoWritableElement;

public abstract class WritableElement extends CaoWritableElement {

	protected Map<String, Object> data = new HashMap<String, Object>();
	
	public WritableElement(CaoElement master) throws CaoException {
		super(master);
	}

	@Override
	public void setBoolean(String name, boolean value) throws CaoException {
		CaoMetaDefinition def = master.getMetadata().getDefinition(name);
		if (def==null||def.getType()!=TYPE.BOOLEAN)
			throw new CaoException("Unknown attribute: " + name);
		setString(name, String.valueOf(value));
	}

	@Override
	public void setCalendar(String name, Calendar value) throws CaoException {
		CaoMetaDefinition def = master.getMetadata().getDefinition(name);
		if (def==null||def.getType()!=TYPE.DATETIME)
			throw new CaoException("Unknown attribute: " + name);
		setString(name, MCast.toString(value));
	}

	@Override
	public void setDate(String name, MDate value) throws CaoException {
		CaoMetaDefinition def = master.getMetadata().getDefinition(name);
		if (def==null||def.getType()!=TYPE.DATETIME)
			throw new CaoException("Unknown attribute: " + name);
		setString(name, value.toString());
	}
	
	@Override
	public void setDouble(String name, double value) throws CaoException {
		CaoMetaDefinition def = master.getMetadata().getDefinition(name);
		if (def==null||def.getType()!=TYPE.DOUBLE)
			throw new CaoException("Unknown attribute: " + name);
		setString(name, MCast.toString(value));
	}

	@Override
	public void setList(String name, CaoList value) throws CaoException {
		CaoMetaDefinition def = master.getMetadata().getDefinition(name);
		if (def==null||def.getType()!=TYPE.LIST)
			throw new CaoException("Unknown attribute: " + name);
		data.put(name, value);
	}

	@Override
	public void setLong(String name, long value) throws CaoException {
		CaoMetaDefinition def = master.getMetadata().getDefinition(name);
		if (def==null||def.getType()!=TYPE.LONG)
			throw new CaoException("Unknown attribute: " + name);
		setString(name, String.valueOf(value));
	}

	@Override
	public void setObject(String name, Object value) throws CaoException {
		CaoMetaDefinition def = master.getMetadata().getDefinition(name);
		if (def==null||def.getType()!=TYPE.OBJECT)
			throw new CaoException("Unknown attribute: " + name);
		data.put(name, value);
	}

	@Override
	public void setString(String name, String value) throws CaoException {
		CaoMetaDefinition def = master.getMetadata().getDefinition(name);
		if (def==null)
			throw new CaoException("Unknown attribute: " + name);
		data.put(name, value);
	}

	@Override
	public boolean getBoolean(String name,boolean def) {
		Object v = data.get(name);
		if (v==null) return master.getBoolean(name,def);
		return MCast.toboolean(v.toString(), false);
	}

	@Override
	public CaoList getChildren(CaoAccess CaoAccess) throws CaoException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Calendar getCalendar(String name){
		Object v = data.get(name);
		if (v==null) return master.getCalendar(name);
		return MCast.toCalendar(v.toString());
	}

	@Override
	public double getDouble(String name,double def) {
		Object v = data.get(name);
		if (v==null) return master.getDouble(name,def);
		return MCast.todouble(v.toString(),0);
	}

	@Override
	public String getId() throws CaoException {
		return master.getId();
	}

	@Override
	public CaoList getList(String name, CaoAccess CaoAccess, String... attributes)
			throws CaoException {
		Object v = data.get(name);
		if (v==null) return master.getList(name,CaoAccess,attributes);
		return (CaoList)v;
	}

	@Override
	public long getLong(String name,long def) {
		Object v = data.get(name);
		if (v==null) return master.getLong(name,def);
		return Long.valueOf(v.toString());
	}

	@Override
	public CaoMetadata getMetadata() {
		return master.getMetadata();
	}

	@Override
	public String getName() throws CaoException {
		return master.getName();
	}

	@Override
	public Object getObject(String name, String... attributes)
			throws CaoException {
		Object v = data.get(name);
		if (v==null) return master.getObject(name);
		return v;
	}

	@Override
	public String getString(String name) throws CaoException {
		Object v = data.get(name);
		if (v==null) return master.getString(name);
		return v.toString();
	}

	@Override
	public CaoWritableElement getWritableNode() {
		return this;
	}

	@Override
	public boolean isNode() {
		return master.isNode();
	}

	@Override
	public boolean isWritable() {
		return master.isWritable();
	}

	@Override
	public void reload() throws CaoException {
		data.clear();
	}

	@Override
	public abstract void save() throws CaoException;

	@Override
	public boolean isLocked(boolean owner) throws CaoException {
		return master.isLocked(owner);
	}

	@Override
	public boolean lock(int timeout) throws CaoException {
		return master.lock(timeout);
	}

	@Override
	public boolean unlock() throws CaoException {
		return master.unlock();
	}

	public boolean isDirty() {
		return data.size() != 0;
	}
}
