package de.mhus.lib.config;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

import de.mhus.lib.logging.Log;

public class DirectoryConfigProvider extends IConfigProvider {

	private static Log log = Log.getLog(DirectoryConfigProvider.class);
	
	private File dir;
	private MConfigFactory factory;

	private boolean createNew;

	public DirectoryConfigProvider(File directory, boolean createNew) {
		this(MConfigFactory.getInstance(),directory,createNew);
	}
	
	public DirectoryConfigProvider(MConfigFactory factory, File directory, boolean createNew) {
		this.factory = factory;
		this.dir = directory;
		this.createNew = createNew;
	}
	
	@Override
	protected IConfig createConfig(String key) throws Exception {
		log.t("loading",dir,key);
		File f = new File(dir,key);
		if (f.exists() && f.isFile()) {
			IConfig config = factory.createConfigForFile(key);
			if (!(config instanceof IConfigRoot))
				return null;
			try {
				FileReader reader = new FileReader(f);
				((IConfigRoot)config).readConfig(reader);
				reader.close();
			} catch (Exception e) {
				log.d(dir,key,e);
			}
		} else
		if (!f.exists() && createNew) {
			IConfig config = factory.createConfigForFile(key);
			return config;
		} else
			log.t("config not found",dir,key);
		return null;
	}

	@Override
	public boolean isWriteable() {
		return dir.canWrite();
	}

	@Override
	protected void writeConfiguration(String name, IConfigRoot config) {
		log.t("writing",dir,name);
		File f = new File(dir,name);
		if (!createNew && !f.exists()) {
			log.d("can't write new config",dir,name);
			return;
		}
		try {
			FileWriter writer = new FileWriter(f);
			config.writeConfig(writer);
			writer.flush();
			writer.close();
		} catch (Exception e) {
			log.d(dir,name,e);
		}
	}

}
