package de.mhus.lib.config;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import de.mhus.lib.MCast;
import de.mhus.lib.MException;

public class HashConfig extends IConfig {

	private HashMap<String, String> 	properties 	= null;
	private HashMap<String, LinkedList<HashConfig>> children	= null;
	private String name;
	private IConfig parent;
	
	public HashConfig() {
		this(null,null);
	}
	
	public HashConfig(String name,IConfig parent) {
		this.name = name;
		this.parent = parent;
		properties = new HashMap<String, String>();
		children = new HashMap<String, LinkedList<HashConfig>>();
	}

	public HashConfig(Map<String, String> fill) {
		this(null,null);
		if (fill !=null) properties.putAll(fill);
	}

	@Override
	public String[] getPropertyKeys() {
		return properties.keySet().toArray(new String[properties.size()]);
	}

	@Override
	public IConfig getConfig(String key) {
		LinkedList<HashConfig> list = children.get(key);
		if (list == null || list.size() == 0) return null;
		return list.getFirst();
	}

	@Override
	public IConfig[] getConfigBundle(String key) {
		LinkedList<HashConfig> list = children.get(key);
		if (list == null || list.size() == 0) return new IConfig[0];
		return list.toArray(new HashConfig[list.size()]);
	}

	@Override
	public IConfig[] getConfigBundle() {
		
		LinkedList<HashConfig> out = new LinkedList<HashConfig>();
		for (Entry<String, LinkedList<HashConfig>> item : children.entrySet()) {
			LinkedList<HashConfig> list = item.getValue();
			for ( HashConfig item2 : list)
				out.add(item2);
		}
		if (out == null || out.size() == 0) return new IConfig[0];
		return out.toArray(new HashConfig[out.size()]);
	}
	
	@Override
	public String[] getConfigKeys() {
		return children.keySet().toArray(new String[children.size()]);
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public IConfig createConfig(String key) throws MException {
		HashConfig child = new HashConfig(key,this);
		LinkedList<HashConfig> list = children.get(key);
		if (list == null) {
			list = new LinkedList<HashConfig>();
			children.put(key, list);
		}
		list.add(child);
		return child;
	}

	@Override
	public int moveConfig(IConfig config, int newPos) throws MException {

		if (!(config instanceof HashConfig))
			throw new MException("not a hashconfig");

		LinkedList<HashConfig> list = children.get(config.getName());
		if (list == null || list.size() == 0 || !list.contains(config))
			throw new MException("config not in my list");
		
		if (list.size() == 1) {
			if (newPos == MOVE_FIRST || newPos == MOVE_LAST || newPos == 0)
				return 0;
			throw new MException("out of range");
		}
		
		if (newPos == MOVE_FIRST) {
			list.remove(config);
			list.add(0, (HashConfig) config);
			return 0;
		}
		
		if(newPos == MOVE_LAST) {
			list.remove(config);
			list.add((HashConfig) config);
			return list.size()-1;
		}
		
		if (newPos == MOVE_DOWN) {
			int pos = list.indexOf(config);
			if (pos == list.size()-1)
				throw new MException("out of range");				
			list.remove(config);
			list.add(pos+1,(HashConfig) config);
			return pos+1;
		}
		
		if (newPos == MOVE_UP) {
			int pos = list.indexOf(config);
			if (pos == 0)
				throw new MException("out of range");				
			list.remove(config);
			list.add(pos-1,(HashConfig) config);
			return pos-1;
		}
		
		if (newPos < 0 || newPos >= list.size())
			throw new MException("out of range");			

		list.remove(config);
		list.add(newPos,(HashConfig) config);
		
		return newPos;
	}

	@Override
	public void removeConfig(IConfig config) throws MException {

		if (!(config instanceof HashConfig)) return;
			
		LinkedList<HashConfig> list = children.get(config.getName());
		if (list == null) return;
		list.remove(config);
		if (list.size() == 0)
			children.remove(list);
	}

	@Override
	public String getProperty(String name) {
		return properties.get(name);
	}

	@Override
	public boolean isProperty(String name) {
		return properties.containsKey(name);
	}

	@Override
	public void removeProperty(String key) throws MException {
		properties.remove(key);
	}

	@Override
	public void setProperty(String key, Object value) throws MException {
		properties.put(key, MCast.objectToString(value));
	}

	@Override
	public boolean isEditable() {
		return true;
	}

	@Override
	public IConfig getParent() {
		return parent;
	}

	@Override
	public Set<String> keys() {
		return properties.keySet();
	}

}
