/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.config;

/**
 * This interface represent a generic configuration. Configurations are
 * sets of definitions. The definitions can be bound together to a inner configuration.
 * 
 * The configuration file syntax is not declared in this interface. The
 * implementation have to interpret the stream.
 * 
 */
import java.io.Reader;
import java.io.Writer;

public interface IConfigRoot {

	/**
	 * Read the configuration from the stream.
	 * 
	 * @param configStream
	 * @throws Exception
	 */
	public void readConfig(Reader configStream) throws Exception;

	/**
	 * Write the configuration to the stream or throw an exception if not supported.
	 * 
	 * @param configStream
	 * @throws Exception
	 */
	public void writeConfig(Writer configStream) throws Exception;

	/**
	 * Return true if the config is changed since the last write operation.
	 * 
	 * @return
	 */
	public boolean isConfigChanged();
	
}
