package de.mhus.lib.config;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.node.ArrayNode;
import org.codehaus.jackson.node.ObjectNode;
import org.codehaus.jackson.node.TextNode;

import de.mhus.lib.MCast;
import de.mhus.lib.MException;

public class JsonConfig extends IConfig {

	private ObjectNode node;
	protected String name;
	private IConfig parent;
	// private int index = -1;
	
	public JsonConfig(String json) throws Exception {
		ObjectMapper mapper = new ObjectMapper();
		JsonNode nodex = mapper.readValue(json, JsonNode.class);
		if (nodex instanceof ObjectNode)
			node = (ObjectNode) nodex;
		else {
			node = ((ArrayNode)nodex).objectNode();
			node.put("default", nodex);
		}
	}
	
	public JsonConfig(ObjectNode node) {
		this(null,null,node);
	}
	
	public JsonConfig(String name, IConfig parent, ObjectNode node) {
		this.node = node;
		this.name = name;
		this.parent = parent;
	}

	public JsonConfig() throws Exception {
		ObjectMapper mapper = new ObjectMapper();
		node = (ObjectNode)mapper.readValue("{}", JsonNode.class);
	}

	public JsonConfig(String name2, JsonConfig jsonConfig, TextNode textNode) throws Exception {
		ObjectMapper mapper = new ObjectMapper();
		node = (ObjectNode)mapper.readValue("{}", JsonNode.class);
		node.put("_", textNode);
	}

	@Override
	public IConfig getConfig(String name) {
		JsonNode child = node.get(name);
		if (child == null || !child.isArray() || child.size() < 1) return null;
		return new JsonConfig(name, this, (ObjectNode)((ArrayNode)child).get(0));
	}

	@Override
	public IConfig[] getConfigBundle(String name) {
		JsonNode child = node.get(name);
		if (child==null || !child.isArray()) return new IConfig[0];
		IConfig[] out = new IConfig[child. size()];
		for (int i = 0; i < child.size(); i++) {
			JsonNode obj = child.get(i);
			if (obj instanceof ObjectNode) {
				out[i] = new JsonConfig(name, this, (ObjectNode)child.get(i));
			} else
			if (obj instanceof org.codehaus.jackson.node.TextNode) {
				try {
					out[i] = new JsonConfig(name, this, (org.codehaus.jackson.node.TextNode)child.get(i));
				} catch (Exception e) {
					log.d(name,e);
				}
			}
			
		}
		return out;
	}

	@Override
	public IConfig[] getConfigBundle() {
		LinkedList<IConfig> out = new LinkedList<IConfig>();
		for ( JsonNode  child : node ) {
			if (child !=null && child.isArray()) {
				for (int i = 0; i < child.size(); i++)
					out.add( new JsonConfig(name, this, (ObjectNode)child.get(i)));
			}
			
		}
		if (out == null || out.size() == 0) return new IConfig[0];
		return out.toArray(new HashConfig[out.size()]);
	}
	
	@Override
	public String[] getConfigKeys() {
		LinkedList<String> out = new LinkedList<String>();
		for (Iterator<String> i = node.getFieldNames(); i.hasNext();) {
			String name = i.next();
			JsonNode child = node.get(name);
			if (child.isArray())
				out.add(name);
		}
		return out.toArray(new String[out.size()]);
	}

	@Override
	public Object getProperty(String name) {
		JsonNode child = node.get(name);
		if (child==null) return null;
		return child.getValueAsText();
	}

	@Override
	public String[] getPropertyKeys() {
		LinkedList<String> out = new LinkedList<String>();
		for (Iterator<String> i = node.getFieldNames(); i.hasNext();) {
			String name = i.next();
			JsonNode child = node.get(name);
			if (!child.isArray())
				out.add(name);
		}
		return out.toArray(new String[out.size()]);
	}

	@Override
	public boolean isProperty(String name) {
		JsonNode child = node.get(name);
		return (child!=null && !child.isArray());
	}

	@Override
	public void removeProperty(String name) {
		getNode().remove(name);
	}

	@Override
	public void setProperty(String name, Object value) {
		getNode().put(name,MCast.objectToString(value));
	}

	public ObjectNode getNode() {
		return node;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public IConfig createConfig(String key) throws MException {
		
		// find array node, to append new config
		if (node.get(key) != null && !node.get(key).isArray()) {
			node.remove(key);
		}
		ArrayNode array = (ArrayNode) node.get(key);
		if (array == null) { 					// if not, create one
			array = node.arrayNode();
			node.put(key, array);
			
		}
		
		if (! (array instanceof ArrayNode) ) {
			throw new MException(key + " is not an array");
		}
		
		// create new object node in array
		ObjectNode out = array.objectNode();
		array.add(out);
		
		return new JsonConfig(key, this, out);
	}

	@Override
	public int moveConfig(IConfig config, int newPos) throws MException {
		if (!(config instanceof JsonConfig))
			throw new MException("not JsonConfig");
		
		// find array node, to append new config
		JsonNode array = node.get(config.getName());
		if (array == null)
			throw new MException("config set not found");
		
		int pos = findPosOf((ArrayNode)array,(JsonConfig)config);
		if (pos < 0)
			throw new MException("could not find child");

		// make it simple if only one element is in list
		if (((ArrayNode)array).size() == 1) {
			if (newPos == MOVE_FIRST || newPos == MOVE_LAST || newPos == 0)
				return 0;
			throw new MException("out of range");
		}

		if (newPos == MOVE_FIRST) {
			((ArrayNode)array).remove(pos);
			((ArrayNode)array).insert(0, ((JsonConfig)config).node);
			return 0;
		}
		if(newPos == MOVE_LAST) {
			((ArrayNode)array).remove(pos);
			((ArrayNode)array).add(((JsonConfig)config).node);
			return ((ArrayNode)array).size()-1;
		}
		
		if (newPos == MOVE_DOWN) {
			if (pos == ((ArrayNode)array).size()-1)
				throw new MException("out of range");
				
			((ArrayNode)array).remove(pos);
			((ArrayNode)array).insert(pos+1, ((JsonConfig)config).node);
			return pos+1;
		}
		
		if (newPos == MOVE_UP) {
			if (pos == 0)
				throw new MException("out of range");

			((ArrayNode)array).remove(pos);
			((ArrayNode)array).insert(pos-1, ((JsonConfig)config).node);
			return pos-1;
		}
		
		if (pos == newPos) return pos;
		
		((ArrayNode)array).remove(pos);
		((ArrayNode)array).insert(newPos, ((JsonConfig)config).node);
		
		return newPos;
	}

	@Override
	public void removeConfig(IConfig config) throws MException {
		
		if (!(config instanceof JsonConfig)) return;
		
		// find array node, to append new config
		JsonNode array = node.get(config.getName());
		if (array == null) return;
		
		int pos = findPosOf((ArrayNode)array,(JsonConfig)config);
		if (pos < 0)
			throw new MException("could not find child");
		((ArrayNode)array).remove( pos );
	}

	protected int findPosOf(ArrayNode array, JsonConfig config) {
		
		//if (config.index >= 0) return config.index;
		
		int cnt = 0;
		for (JsonNode item : array) {
			if (item == config.node)
				return cnt;
			cnt++;
		}
		
		return -1;
	}

	@Override
	public boolean isEditable() {
		return true;
	}

	@Override
	public IConfig getParent() {
		return parent;
	}
	
	public void write(OutputStream os) throws IOException {
		ObjectMapper m = new ObjectMapper();
		m.writeValue(os,node);
	}
	
	public void write(Writer os) throws IOException {
		ObjectMapper m = new ObjectMapper();
		m.writeValue(os,node);
	}

}
