package de.mhus.lib.config;

import java.util.Calendar;

import de.mhus.lib.MDate;
import de.mhus.lib.MException;
import de.mhus.lib.util.ReadOnlyException;

/**
 * This config implementation is a proxy to hidde the write functionality of another
 * config implementation.
 * 
 * @author mikehummel
 *
 */
public class ReadOnlyConfig extends IConfig {

	private IConfig instance;
	private ReadOnlyConfig parent;

	public ReadOnlyConfig(IConfig instance) {
		this(null,instance);
	}
	
	public ReadOnlyConfig(ReadOnlyConfig parent, IConfig instance) {
		this.parent = parent;
		this.instance = instance;
	}

	public Object getProperty(String name) {
		return instance.getProperty(name);
	}

	@SuppressWarnings("deprecation")
	@Override
	public int getProperty(String name, int def) {
		return instance.getProperty(name, def);
	}

	@SuppressWarnings("deprecation")
	@Override
	public boolean getProperty(String name, boolean def) {
		return instance.getProperty(name, def);
	}

	@SuppressWarnings("deprecation")
	@Override
	public String getProperty(String name, String def) {
		return instance.getProperty(name, def);
	}

	@Override
	public String getString(String name, String def) {
		return instance.getString(name, def);
	}

	@Override
	public boolean getBoolean(String name, boolean def) {
		return instance.getBoolean(name, def);
	}

	@Override
	public int getInt(String name, int def) {
		return instance.getInt(name, def);
	}

	@Override
	public long getLong(String name, long def) {
		return instance.getLong(name, def);
	}

	@Override
	public float getFloat(String name, float def) {
		return instance.getFloat(name, def);
	}

	@Override
	public String[] getPropertyKeys() {
		return instance.getPropertyKeys();
	}

	@Override
	public double getDouble(String name, double def) {
		return instance.getDouble(name, def);
	}

	@Override
	public Calendar getCalendar(String name) {
		return instance.getCalendar(name);
	}

	@Override
	public IConfig getConfig(String key) {
		return new ReadOnlyConfig(this,instance.getConfig(key));
	}

	@Override
	public MDate getDate(String name) {
		return instance.getDate(name);
	}

	@Override
	public int hashCode() {
		return instance.hashCode();
	}

	@Override
	public void setInt(String name, int value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public IConfig[] getConfigBundle() {
		IConfig[] out = instance.getConfigBundle();
		if (out == null) return null;
		for (int i = 0; i < out.length; i++)
			out[i] = new ReadOnlyConfig(this, out[i]);
		return out;
	}

	@Override
	public void setLong(String name, long value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public void setDouble(String name, double value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public IConfig[] getConfigBundle(String key) {
		IConfig[] out = instance.getConfigBundle(key);
		if (out == null) return null;
		for (int i = 0; i < out.length; i++)
			out[i] = new ReadOnlyConfig(this, out[i]);
		return out;
	}

	@Override
	public void setFloat(String name, float value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public void setBoolean(String name, boolean value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public String[] getConfigKeys() {
		return instance.getConfigKeys();
	}

	@Override
	public void setCalendar(String name, Calendar value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public String getName() {
		return instance.getName();
	}

	@Override
	public void setDate(String name, MDate value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public IConfig createConfig(String key) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public boolean isProperty(String name) {
		return instance.isProperty(name);
	}

	@Override
	public void removeProperty(String key) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public int moveConfig(IConfig config, int newPos) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public void setProperty(String key, Object value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public boolean isEditable() {
		return false;
	}

	@Override
	public void removeConfig(IConfig config) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public void setUserObject(Object userObject) {
		super.setUserObject(userObject);
	}

	@Override
	public boolean equals(Object obj) {
		return instance.equals(obj);
	}

	@Override
	public Object getUserObject() {
		return super.getUserObject();
	}

	@Override
	public String getExtracted(String key) {
		return instance.getExtracted(key);
	}

	@Override
	public String getExtracted(String key, String def) {
		return instance.getExtracted(key, def);
	}

	@Override
	public void setString(String name, String value) throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public IConfig getParent() {
		return parent != null ? parent : new ReadOnlyConfig(instance.getParent());
	}

	@Override
	public boolean canSave() {
		return false;
	}

	@Override
	public void save() throws MException {
		throw new ReadOnlyException();
	}

	@Override
	public String toString() {
		return instance.toString();
	}


}
