/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.form;

/**
 * If a form should not be managed direct by calling getters / setters implement
 * this interface to manage the form by hand. Instead of calling the getters /
 * setters the system will call the functions of this interface. Managed forms
 * are usefull for dynamic form values.
 * 
 * @author mike
 * 
 */
public interface IFormDynamic extends MForm {

	/**
	 * Set the value of a attribute.
	 * 
	 * @param name
	 *            Unique attribute name
	 * @param value
	 *            Value to be set
	 * @throws FormException
	 *             Throw this if the attribute is not managed
	 */
	public void setFormValue(String name, Object value) throws FormException;

	/**
	 * Get the value of an attribute. It's not recommended to implement this for
	 * all or any attribute. If you don't want to allow get value throw the
	 * empty exception.
	 * 
	 * @param name
	 *            Name of the attribute
	 * @return The corresponding value.
	 * @throws FormException
	 *             Throw this if the attribute is unknown or you dont want to
	 *             support it.
	 */
	public Object getFormValue(String name) throws FormException;

	/**
	 * This function should throw an exception if the attribute is unknown to
	 * set.
	 * 
	 * @param name
	 *            name of the attribute
	 * @param class1
	 *            type of the attribute
	 * @throws FormException
	 *             Throw the exception if the attribute can't be set.
	 */
	public void validateFormAttribute(String name, Class<?> class1)
			throws FormException;

}
