package de.mhus.lib.form;

import java.util.Collection;
import java.util.TreeMap;

import de.mhus.lib.MActivator;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.form.builders.FormElementRegistry;
import de.mhus.lib.form.objects.FGroup;
import de.mhus.lib.form.objects.FObject;
import de.mhus.lib.form.objects.ObjectList;
import de.mhus.lib.form.objects.FObject.WHAT_CHANGED;
import de.mhus.lib.resources.MResources;
import de.mhus.lib.util.MNls;

/**
 * The main class to hold the model.
 * 
 * A model have the following parts:
 * - main definition: The general definition
 * - element definitions: This is the definition of the parameters, it comes from annotations of the target object
 * - rules definition
 * - view definition
 * 
 * TODO add repeating block support
 * 
 * @author mike
 * 
 */
public class MFormModel extends FObject {

	public static enum RULE_ACTION {
		ERROR, WARN, DISABLE
	}

	public static enum TYPE {
		TEXT, TEXTAREA, PASSWORD, BOOLEAN, SELECT, EMAIL, FILE, HTML, HIDDEN, DATE, NUMBER, BINARY
	}
	
	
	public static final String ELEMENT_REGISTRY = FormElementRegistry.class.getCanonicalName();

	private ObjectList list = new ObjectList();
	private MActivator activator;
	private MFormLayout layout;
	private MNls properties;

	private MResources resourceProvider;

	private MForm target;

	public MFormModel(MActivator activator, MForm target) {
		this.activator = activator;
		setTarget(target);
	}

	public void setTarget(MForm target) {
		this.target = target;
		for (FObject f : list)
			f.fireChanged(this, WHAT_CHANGED.TARGET);
		fireChanged(this, WHAT_CHANGED.TARGET);
	}

	public MForm getTarget() {
		return target;
	}
	
	public void setEnabled(boolean enabled) {
		if (isLocalEnabled() == enabled)
			return;

		super.setEnabled(enabled);
		
		for (FObject f : list)
			f.fireEnabledChanged();
		
	}
	
	public ObjectList getList() {
		return list;
	}

	@Override
	public Object getFormValue() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	protected Class<?> getFormValueClass() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	protected void setFormValue(Object value) {
		// TODO Auto-generated method stub

	}

	public void setLayout(MFormLayout formLayout) {
		this.layout = formLayout;
	}
	
	public MFormLayout getLayout() {
		return layout;
	}

	public FObject getElement(String elementName) {
		// for ( FObject obj : list )
		// if ( elementName.equals( obj.getName() ) )
		// return obj;
		// return null;
		if (elementName.equals("@"))
			return this;
		String[] parts = elementName.split("/");
		return getElement(parts, 0, list);
	}

	private FObject getElement(String[] elementName, int pos, ObjectList l) {
		if (pos + 1 >= elementName.length)
			return l.getByName(elementName[pos]);
//		if (elementName[pos].startsWith("@")) {
//			String fName = elementName[pos].substring(1);
//			for (MFormModel f : pages) {
//				if (f.getName().equals(fName))
//					return f.getElement(elementName, pos + 1, ((FGroup) l
//							.getByName(elementName[pos])).getList());
//			}
//			return null;
//		}
		return getElement(elementName, pos + 1, ((FGroup) l
				.getByName(elementName[pos])).getList());
	}


//	public void validateTarget() throws FormException {
//		for (Iterator<FObject> i = getList().iterator(); i.hasNext();) {
//			FObject obj = i.next();
//			obj.targetValidate();
//		}
//	}

	public boolean isValide(boolean checkWarnings) {
		if (!super.isValide(checkWarnings))
			return false;

		for (FObject f : list)
			if (!f.isValide(checkWarnings))
				return false;

//		if (pages == null)
//			return true;
//		for (MFormModel f : pages)
//			if (!f.isValide(checkWarnings))
//				return false;

		return true;
	}

	public MActivator getActivator() {
		return activator;
	}
	
	public MResources getResourceProvider() {
		return resourceProvider;
	}
	
	public void setResourceProvider(MResources in) {
		resourceProvider = in;
	}
	
	public void setResourceName(String resourceName) throws Exception {
		if (resourceProvider == null)
			properties = new MNls(getActivator(),getTarget().getClass(),resourceName);
		else
			properties = new MNls(getResourceProvider(),getTarget().getClass(),resourceName);
	}

	public void setResourceBundle(MNls res) {
		properties = res;
	}
	
	public synchronized MNls getNls(){
		if (properties==null) {
			try {
				setResourceName(null);
			} catch (Exception e) {
				log().i(e);
			}
		}
		return properties;
	}
	
	public void fireStatusEvents() {

		super.fireStatusEvents();

		for (FObject f : list)
			f.fireStatusEvents();

//		if (pages == null)
//			return;
//		for (MFormModel f : pages)
//			f.fireStatusEvents();

	}

	public Collection<FObject> getSortedList() {
		
		TreeMap<Integer, FObject> sorted = new TreeMap<Integer, FObject>();
		
		for ( FObject object : getList() ) {
			int id = object.getSortId();
			while (sorted.containsKey(id)) id++;
			sorted.put(id, object);
		}

		return sorted.values();
	}

	@Override
	public void initWithConfig(MFormModel form2, IConfig config) throws FormException {
		// TODO Auto-generated method stub
		
	}
	
	public void loadFromTarget() {
		for ( FObject object : getList() )
				object.loadFromTarget();
	}
	
	public void saveToTarget(boolean force) throws FormException {
		for ( FObject object : getList() )
			object.saveToTarget(force);
	}

	public String toString() {
		return list != null ? list.toString() : "?";
	}
}
