package de.mhus.lib.form.builders;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import de.mhus.lib.MXml;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.config.XmlConfig;
import de.mhus.lib.form.MFormLayout;
import de.mhus.lib.form.MFormModel;
import de.mhus.lib.form.layout.LComposite;
import de.mhus.lib.form.layout.LField;
import de.mhus.lib.form.layout.LGroup;
import de.mhus.lib.form.layout.LObject;
import de.mhus.lib.form.layout.LPage;
import de.mhus.lib.form.layout.LTabbed;

public class FormLayoutBuilder extends MFormLayout {
	
	public FormLayoutBuilder(MFormModel model, IConfig layout) {
		this(model);
		load(layout,root);
	}

	protected FormLayoutBuilder(MFormModel model) {
		super(model);
	}
	
	public FormLayoutBuilder(MFormModel model, String resource) throws ParserConfigurationException, SAXException, IOException {
		this(model);
		String res = model.getTarget().getClass().getCanonicalName().replace('.', '/');
		int pos = res.lastIndexOf('/');
		if (pos > 0)
			res = res.substring(0, pos + 1);
		res = res + resource;
		// log.info("Load Resource: " + res);
		InputStream is = model.getTarget().getClass().getClassLoader()
				.getResourceAsStream(res);
		IConfig document = new XmlConfig(MXml.loadXml(is).getDocumentElement());
		load(document,root);
	}

	protected void load(IConfig layout, LComposite composite) {

		for ( IConfig element : layout.getConfigBundle("element") ) {

			LObject next = null;
			String type = element.getString("type", "field");
			if (type.equals("field")) {
				next = new LField();
			} else
			if (type.equals("page")) {
				next = new LPage();
				load(element,(LComposite) next);
			} else
			if (type.equals("tabbed")) {
				next = new LTabbed();
				load(element,(LComposite) next);
				for ( IConfig element2 : element.getConfigBundle("element") ) {
					LComposite next2 = new LComposite();
					next2.setConfig(element2);
					next2.setModel(this, model);
					((LComposite)next).add(next2);
					load(element2,next2);
				}
			} else
			if (type.equals("group")) {
				next = new LGroup();
				load(element,(LComposite) next);
			}
			
			if (next != null) {
				next.setModel(this,model);
				next.setConfig(element);
				next.init();
				composite.add(next);
			}
			
		}
		
	}


}
