/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.form.objects;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.form.FormException;
import de.mhus.lib.form.MFormModel;

/**
 * Represent a boolean value. Needed function is set[Name]( boolean in ). For
 * read access also implement is[Name]().
 * 
 * @author mike
 * 
 */
public class FBoolean extends FObject {

	private boolean value;
	private boolean defaultValue;

	public boolean isValue() {
		return value;
	}

	public void setValue(boolean value) {
		if (value == this.value)
			return;
		this.value = value;
		setChanged(true);
	}

	public boolean isDefaultValue() {
		return defaultValue;
	}

	public void setDefaultValue(boolean defaultValue) {
		this.defaultValue = defaultValue;
	}

	@Override
	public Object getFormValue() {
		return isValue();
	}

	@Override
	protected Class<?> getFormValueClass() {
		return boolean.class;
	}

	@Override
	protected void setFormValue(Object value) {
		setValue(((Boolean) value).booleanValue());
	}

	public void initWithConfig(MFormModel form2, IConfig config) throws FormException {
		setValue(config.getBoolean("value", false));
	}
}
