/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.form.objects;

import java.util.Date;

import de.mhus.lib.MCast;
import de.mhus.lib.MDate;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.form.FormException;
import de.mhus.lib.form.MFormModel;

/**
 * Represent a boolean value. Needed function is set[Name]( boolean in ). For
 * read access also implement is[Name]().
 * 
 * @author mike
 * 
 */
public class FDate extends FObject {

	private Date value;
	private Date defaultValue = null; //TODO .... this is not ok

	public Date getValue() {
		return value;
	}

	public void setValue(Date value) {
		if ( (value == null && this.value == null) || (value != null && this.value !=null && value.equals(this.value)) )
			return;
		this.value = value;
		setChanged(true);
	}

	@Override
	public Object getFormValue() {
		return getValue();
	}

	@Override
	protected Class<?> getFormValueClass() {
		return Date.class;
	}

	@Override
	protected void setFormValue(Object value) {
		if (value == null) setValue(defaultValue);
		if (value instanceof Date) setValue((Date)value);
		setValue(MCast.toDate(value.toString(), defaultValue));
	}

	public void initWithConfig(MFormModel form2, IConfig config) throws FormException {
		try {
			MDate date = config.getDate("value");
			setValue(date.toDate());
		} catch (Throwable t) {
			setValue(defaultValue);
		}
	}
}
