/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.form.objects;

import java.util.Iterator;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.form.FormException;
import de.mhus.lib.form.MFormModel;
import de.mhus.lib.form.objects.FObject.WHAT_CHANGED;

/**
 * Represent a grouped set of attributes. This class and subclasses will have
 * only effect to the output presentation. It don't set values or change the
 * names of the children. If you wan't you can implement the function boolean
 * is[Name]Expanded() and set[Name]Expanded(boolean) to register and remember
 * the status of the group. This maybe can change also your behavior.
 * 
 * @author mike
 * 
 */
public class FGroup extends FBoolean {

	private ObjectList list = new ObjectList();

	public ObjectList getList() {
		return list;
	}

	@Override
	public void loadFromTarget() {

		super.loadFromTarget();

		for (Iterator<FObject> i = getList().iterator(); i.hasNext();) {
			FObject obj = i.next();
			obj.loadFromTarget();
		}
	}
	
	@Override
	public void saveToTarget(boolean force) throws FormException {

		try {
			super.saveToTarget(force);
		} catch (FormException e) {
			log().warn(getClass().getName() + " : " + getName() + ": "
					+ e.toString());
		}

		for (Iterator<FObject> i = getList().iterator(); i.hasNext();) {
			FObject obj = i.next();
			obj.saveToTarget(force);
		}
	}

//	@Override
//	public void targetValidate() throws FormException {
//
//		for (Iterator<FObject> i = getList().iterator(); i.hasNext();) {
//			FObject obj = i.next();
//			obj.targetValidate();
//		}
//
//	}

	@Override
	protected String getNameAsMethodSuffix() {
		if (getName() == null || getName().length() < 1)
			return "Expanded";
		return getName().substring(0, 1).toUpperCase() + getName().substring(1)
				+ "Expanded";
	}

	public void initWithConfig(MFormModel form2, IConfig config) throws FormException {
		super.initWithConfig(form2, config);
	}
	
	public void fireEnabledChanged() {
		super.fireEnabledChanged();
		for (FObject f : list)
			f.fireEnabledChanged();
	}
	
}
