package de.mhus.lib.form.objects;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.form.FormException;
import de.mhus.lib.form.MFormModel;

public class FNumber extends FObject {

	private Number value;
	private Class<?> clazz;
	private String type;
	private Number min = null;
	private Number max = null;
	private Number steps = null;
	
	@Override
	protected void setFormValue(Object value) throws FormException {
		try {
			if (value == null || value.toString().length()==0)
				this.value = new Double(min.doubleValue());
			else
			if (value instanceof Number)
				this.value = (Number) value;
			else
				value = Double.parseDouble(value.toString());
		} catch (Throwable e) {
			this.value = new Double(min.doubleValue());
		}
	}

	@Override
	public Object getFormValue() {
		if (value == null) value = min;
		
		if (clazz == int.class)
			return value.intValue();
		if (clazz == long.class)
			return value.longValue();
		if (clazz == double.class)
			return value.doubleValue();
		if (clazz == float.class)
			return value.floatValue();
		
		return value;
	}

	@Override
	protected Class<?> getFormValueClass() {
		return clazz;
	}

	public Number getMinimum() {
		return min;
	}
	
	public Number getMaximum() {
		return max;
	}
	
	public Number getSteps() {
		return steps;
	}
	
	public Number getValue() {
		return value;
	}
	
	public void setValue(Number in) {
		if (in == null) return;
		value = in;
	}
	
	
	public void initWithConfig(MFormModel form2, IConfig config) throws FormException {
		
		
		clazz = Number.class;
		type = config.getName();
		if (type == null) type = config.getString("type", "number");
		if (config.getName().equals("number"))
			clazz = Number.class;
		else
		if (config.getName().equals("integer"))
			clazz = int.class;
		else
		if (config.getName().equals("long"))
			clazz = long.class;
		else
		if (config.getName().equals("double"))
			clazz = double.class;
		else
		if (config.getName().equals("float"))
			clazz = float.class;

		//
		min = new Double(0);
		max = new Double(Integer.MAX_VALUE);
		steps = null;
		
		if (config.isProperty("min"))
			min = new Double(config.getDouble("min", 0));
		
		if (config.isProperty("max"))
			max = new Double(config.getDouble("max", 0));
				
		if (config.isProperty("steps"))
			steps = new Double(config.getDouble("steps", 0));
		
	}
}
