/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.form.objects;

import de.mhus.lib.form.FormException;
import de.mhus.lib.form.IFormDynamic;
import de.mhus.lib.form.objects.FObject.WHAT_CHANGED;

/**
 * With this input you can handle a option list. See FString for details. The
 * value is one of optionValues or null or a empty string.
 * 
 * @author mike
 * 
 */
public class FOptions extends FString {

	private IDynOptionsProvider dataProvider;

	private String[] optionValues;
	private String[] optionTitles;

	public void setValue(int nr) throws FormException {
		if (nr < 0 || optionValues == null || nr >= optionValues.length) {
			setValue(null);
			return;
		}
		setValue(optionValues[nr]);
	}

	public int getValueNr() {
		if (optionValues == null)
			return -1;
		for (int i = 0; i < optionValues.length; i++)
			if (optionValues[i].equals(getValue()))
				return i;
		return -1;
	}

	public String[] getOptionValues() {
		return optionValues;
	}

	public void setOptionValues(String[] optionValues) {
		this.optionValues = optionValues;
	}

	public String[] getOptionTitles() {
		return optionTitles;
	}

	public void setOptionTitles(String[] optionTitles) {
		this.optionTitles = optionTitles;
	}

	@Override
	public void loadFromTarget() {
		super.loadFromTarget();
		loadOptionsData();
	}
	
	protected void loadOptionsData() {
		dataProvider = null;
		try {
			if (getTarget() instanceof IFormDynamic) {
				setOptionValues((String[]) ((IFormDynamic) getTarget())
						.getFormValue(getName() + ".values"));
				setOptionTitles((String[]) ((IFormDynamic) getTarget())
						.getFormValue(getName() + ".titles"));
				return;
			}
		} catch (Throwable e) {
			log().w(getName(),"Managed",e);
			return;
		}

		if ( dataProvider == null ) {
			try {
				dataProvider = (IDynOptionsProvider)getTarget().getClass().getMethod(
						"get" + getNameAsMethodSuffix() + "DataProvider", new Class[] {})
						.invoke(getTarget(), new Object[] {});
				
			} catch (Throwable e) {
				log().w(getName(),e);
			}
		}
		if ( dataProvider != null ) {
			setOptionValues( dataProvider.getOptionValues() );
			setOptionTitles( dataProvider.getOptionTitles() );
			log().d(getName(),optionValues,optionTitles);
		}		
	}
	
	public void fireChanged(FObject obj, WHAT_CHANGED what) {
		if (what == WHAT_CHANGED.TARGET)
			loadOptionsData();
		super.fireChanged(obj, what);
	}
}
