/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.form.objects;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.form.FormException;
import de.mhus.lib.form.MFormModel;
import de.mhus.lib.form.ValidateException;

public class FString extends FObject {

	private String value;
	private String defaultValue;
	private int min = 0;
	private int max = 0;

	public String getValue() {
		return value;
	}

	public void setValue(String value) throws FormException {
		if (value == null)
			value = "";

		if (value.length() < min)
			throw new ValidateException(getTarget(), this, "min");
		
		if (max > 0 && value.length() > max)
			throw new ValidateException(getTarget(), this, "max");
		
		if ((this.value != null && !this.value.equals(value))
				|| (value != null && !value.equals(this.value))) {
			this.value = value;
			setChanged(true);
		}
	}

	public String getDefaultValue() {
		return defaultValue;
	}

	public void setDefaultValue(String defaultValue) {
		this.defaultValue = defaultValue;
	}

	@Override
	public Object getFormValue() {
		return getValue();
	}

	@Override
	protected Class<?> getFormValueClass() {
		return String.class;
	}

	@Override
	protected void setFormValue(Object value) throws FormException {
		if (value == null)
			setValue(null);
		else
			setValue(String.valueOf( value ));
	}

	public void setMin(int min) {
		this.min = min;
	}

	public int getMin() {
		return min;
	}

	public void setMax(int max) {
		this.max = max;
	}

	public int getMax() {
		return max;
	}

	@Override
	public void initWithConfig(MFormModel form2, IConfig config) throws FormException {
		setValue(config.getString("value",""));
		setMin(config.getInt("min", 0));
		setMax(config.getInt("max", 0));
	}

}
