/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.form.objects;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * Helper to handle the model in a generic way.
 * 
 * @author mike
 * 
 */
public class ObjectList implements Iterable<FObject> {

	private LinkedList<FObject> list = new LinkedList<FObject>();
	private HashMap<String, FObject> map = new HashMap<String, FObject>();

	public boolean add(FObject o) {
		map.put(o.getName(), o);
		return list.add(o);
	}

	public void add(int index, FObject element) {
		map.put(element.getName(), element);
		list.add(index, element);
	}

	public void addFirst(FObject o) {
		map.put(o.getName(), o);
		list.addFirst(o);
	}

	public void addLast(FObject o) {
		map.put(o.getName(), o);
		list.addLast(o);
	}

	public void clear() {
		map.clear();
		list.clear();
	}

	public FObject get(int index) {
		return list.get(index);
	}

	public FObject getFirst() {
		return list.getFirst();
	}

	public FObject getLast() {
		return list.getLast();
	}

	public Iterator<FObject> iterator() {
		return list.iterator();
	}

	public FObject remove(int index) {
		map.remove(list.get(index).getName());
		return list.remove(index);
	}

	public boolean remove(FObject o) {
		map.remove(o.getName());
		return list.remove(o);
	}

	public FObject removeFirst() {
		map.remove(list.getFirst().getName());
		return list.removeFirst();
	}

	public FObject removeLast() {
		map.remove(list.getLast().getName());
		return list.removeLast();
	}

	public FObject set(int index, FObject element) {
		map.put(element.getName(), element);
		return list.set(index, element);
	}

	public int size() {
		return list.size();
	}

	public FObject[] toArray() {
		return list.toArray(new FObject[size()]);
	}

	public FObject getByName(String name) {
		return map.get(name);
	}

	public String toString() {
		return list != null ? list.toString() : "?";
	}
	
}
