package de.mhus.lib.form.ui;

import java.util.LinkedList;
import java.util.List;
import java.util.Observer;

import javax.xml.parsers.ParserConfigurationException;

import de.mhus.lib.MActivator;
import de.mhus.lib.MSingleton;
import de.mhus.lib.form.FormException;
import de.mhus.lib.form.MForm;
import de.mhus.lib.form.MFormLayout;
import de.mhus.lib.form.MFormModel;
import de.mhus.lib.form.builders.FormLayoutSimpleBuilder;
import de.mhus.lib.form.builders.FormModelBuilder;
import de.mhus.lib.lang.MObject;

public class UiModel extends MObject {

	private MFormModel form;
	private MFormLayout layout;
	private LinkedList<UiInput> elements = new LinkedList<UiInput>();
	private Observer descriptionObserver = null;

	public UiModel(MForm target, String schema) throws Exception {
		this(MSingleton.instance().getActivator(), target, schema);
	}
	
	public UiModel(MActivator activator, MForm target, String schema) throws Exception {
		this(new FormModelBuilder(activator, target, schema).getForm());
	}
	
	public UiModel(MFormModel form) {
		this.form = form;
		
		//MFormModel model = form.getLayout().getModel();
		MFormModel model = form;
		if (model.getLayout() == null)
			try {
				model.setLayout(new FormLayoutSimpleBuilder(model));
			} catch (ParserConfigurationException e) {
				log().e(e);
			}
		layout = model.getLayout();
		
	}
	
	public MFormModel getFormModel() {
		return form;
	}

	public MFormLayout getLayoutModel() {
		return layout;
	}
	
	public List<UiInput> getElements() {
		return elements;
	}

	public void transferToForm() {
		for (UiInput x : elements ) {
			try {
				x.setValue();
			} catch (FormException e) {
				log().d(x,e);
			}
		}
	}

	public void transferToUI() {
		for (UiInput x : elements ) {
			try {
				x.getValue();
			} catch (FormException e) {
				log().d(x,e);
			}
		}
	}

	public void dispose() {
		for (UiInput inp : elements ) {
			inp.dispose();
		}
	}
	
	public static interface DescriptionListener {

		void descriptionChanged(boolean isError, String description);

	}

//	private class DescriptionEventHandler extends
//			MEventHandler<DescriptionListener> {
//
//		void fireDescriptionChanged(boolean isError, String description) {
//			for (DescriptionListener entry : getListeners())
//				entry.descriptionChanged(isError,description);
//		}
//
//	}

	public void fireDescriptionChanged(boolean error, String msg) {
		if (descriptionObserver != null)
			try {
				if (!error)
					descriptionObserver.update(null, msg);
			} catch (Throwable t) {
				log().w(t);
			}
	}

	public Observer getDescriptionObserver() {
		return descriptionObserver;
	}

	public void setDescriptionObserver(Observer descriptionObserver) {
		this.descriptionObserver = descriptionObserver;
	}

}
