package de.mhus.lib.form.ui;

import de.mhus.lib.form.MForm;
import de.mhus.lib.form.MFormLayout;
import de.mhus.lib.form.MFormModel;
import de.mhus.lib.form.layout.LComposite;
import de.mhus.lib.form.layout.LField;
import de.mhus.lib.form.layout.LGroup;
import de.mhus.lib.form.layout.LObject;
import de.mhus.lib.form.layout.LPage;
import de.mhus.lib.form.layout.LTabbed;
import de.mhus.lib.form.objects.FBinary;
import de.mhus.lib.form.objects.FBoolean;
import de.mhus.lib.form.objects.FFile;
import de.mhus.lib.form.objects.FNumber;
import de.mhus.lib.form.objects.FObject;
import de.mhus.lib.form.objects.FOptions;
import de.mhus.lib.form.objects.FPassword;
import de.mhus.lib.form.objects.FString;
import de.mhus.lib.form.objects.FStringList;
import de.mhus.lib.form.objects.FTextArea;
import de.mhus.lib.form.objects.FHtml;
import de.mhus.lib.lang.MObject;

public abstract class UiModelBuilder extends MObject {

	private UiModel model;
	
	public UiModelBuilder() {
	}
	
	public UiModelBuilder(MForm target, String schema) throws Exception {
		model = new UiModel(target, schema);
	}

	public UiModelBuilder(MFormModel form) {
		model = new UiModel(form);
	}
	
	public UiModel build() {
		MFormLayout layout = model.getLayoutModel();
		doStart(layout);
		createPartControl(layout.getRoot());
		doEnd(layout);
		return model;
	}

	
	protected void createPartControl(LComposite list) {
		for (LObject object : list) {
			if (object instanceof LTabbed) {
				doTabStart(object);
				for (LObject o : (LComposite)object) {
					if (o instanceof LComposite) {
						doPreTab(object,o);
						createPartControl((LComposite)o);
						doPostTab(object,o);
					}
				}
				doTabEnd(object);
			} else
			if (object instanceof LPage) {
				doPageStart(object);
				for (LObject o : (LComposite)object) {
					if (o instanceof LComposite) {
						doPrePage(object,o);
						createPartControl((LComposite)o);
						doPostPage(object,o);
					}
				}
				doPageEnd(object);
			} else
			if (object instanceof LGroup) {
				doGroupStart(object);
				createPartControl((LComposite)object);
				doGroupEnd(object);
			} else
			if (object instanceof LComposite) {
				doCompositeStart(object);
				for (LObject o : (LComposite)object) {
					if (o instanceof LComposite) {
						doPreComposite(object,o);
						createPartControl((LComposite)o);
						doPostComposite(object,o);
					}
				}
				doCompositeEnd(object);
			} else
			if (object instanceof LField) {
				FObject[] objs = ((LField) object).getObjects();
				if (objs.length == 1) {
					FObject obj = objs[0];
					if (obj instanceof FFile) {
						doFile((LField)object,obj);
					} else if (obj instanceof FHtml) { 
						doHtml((LField)object,obj);
					} else if (obj instanceof FTextArea) {
						doTextArea((LField)object,obj);
					} else if (obj instanceof FOptions) {
						doOptions((LField)object,obj);
					} else if (obj instanceof FPassword) {
						doPassword((LField)object,obj);
					} else if (obj instanceof FBoolean) {
						doBoolean((LField)object,obj);
					} else if (obj instanceof FString) {
						doString((LField)object,obj);
					} else if (obj instanceof FNumber) {
						doNumber((LField)object,obj);
					} else if (obj instanceof FStringList) {
						doStringList((LField)object,obj);
					} else if (obj instanceof FBinary) {
						doBinary((LField)object,obj);
					} else {
						doUnknownField((LField)object,obj);
					}
				} else {
					doMultipleField((LField)object);
				}
			}
		}
	}
	
	
	protected abstract void doEnd(MFormLayout layout);

	protected abstract void doStart(MFormLayout layout);

	protected abstract void doMultipleField(LField object);

	protected abstract void doUnknownField(LField object,FObject field);

	protected abstract void doStringList(LField object,FObject field);

	protected abstract void doNumber(LField object,FObject field);

	protected abstract void doString(LField object,FObject field);

	protected abstract void doBoolean(LField object,FObject field);

	protected abstract void doPassword(LField object,FObject field);

	protected abstract void doOptions(LField object,FObject field);

	protected abstract void doTextArea(LField object,FObject field);

	protected abstract void doHtml(LField object, FObject obj);

	protected abstract void doFile(LField object,FObject field);

	protected abstract void doBinary(LField object, FObject obj);

	protected abstract void doCompositeEnd(LObject object);

	protected abstract void doPostComposite(LObject object, LObject o);

	protected abstract void doPreComposite(LObject object, LObject o);

	protected abstract void doCompositeStart(LObject object);

	protected abstract void doGroupEnd(LObject object);

	protected abstract void doGroupStart(LObject object);

	protected abstract void doPageEnd(LObject object);

	protected abstract void doPostPage(LObject object, LObject o);

	protected abstract void doPrePage(LObject object, LObject o);

	protected abstract void doPageStart(LObject object);

	protected abstract void doTabEnd(LObject object);

	protected abstract void doPostTab(LObject object, LObject o);

	protected abstract void doPreTab(LObject object, LObject o);

	protected abstract void doTabStart(LObject object);


	public void setRawModel(UiModel model) {
		this.model = model;
	}


	public UiModel getModel() {
		return model;
	}
	
}
