package de.mhus.lib.logging;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Observable;

import de.mhus.lib.MSingleton;
import de.mhus.lib.config.IConfig;

public class ConsoleFactory extends Log {

	//public static boolean tracing = true;
	private String name;
	private PrintStream out;
	private LEVEL level = LEVEL.INFO;
	private boolean traces = true;

	public ConsoleFactory() {
		out = System.out;
	}
	
	public ConsoleFactory(IConfig config) throws FileNotFoundException {
		//tracing = config.getBoolean("tracing", true);
		name = config.getExtracted("name","");
		String newLevel = config.getExtracted("level",null);
		if (newLevel != null) level = LEVEL.valueOf(newLevel.toUpperCase());
		
		String file = config.getExtracted("file",null);
		String io = config.getExtracted("stream",null);
		traces    = config.getBoolean("traces", true);
		if (file != null) {
			FileOutputStream f = new FileOutputStream(file,config.getBoolean("append", true));
			out = new PrintStream(f,true);
		} else
		if ("err".equals(io))
		{
			out = System.err;
		} else {
			out = System.out;
		}
	}
	
	public ConsoleFactory(String name,PrintStream stream) {
		this.name = name;
		this.out = stream;
	}

	@Override
    public void update(Observable o, Object arg) {
        level = LEVEL.valueOf(MSingleton.instance().getConfig().getString("LEVEL",level.name()).toUpperCase()); 
        super.update(o, arg);
	}

	@Override
	public void debug(Object message) {
		if (!isDebugEnabled()) return;
		out.println("DEBUG: " + name + " " + message);
		if (message != null && message instanceof Throwable)
			((Throwable)message).printStackTrace(out);
	}

	@Override
	public void debug(Object message, Throwable t) {
		if (!isDebugEnabled()) return;
		out.println("DEBUG: " + name + " " + message);
		if (t!=null && traces) t.printStackTrace(out);
	}

	@Override
	public void error(Object message) {
		if (!isErrorEnabled()) return;
		out.println("ERROR: " + name + " " + message);
		if (message != null && message instanceof Throwable && traces)
			((Throwable)message).printStackTrace(out);
	}

	@Override
	public void error(Object message, Throwable t) {
		if (!isErrorEnabled()) return;
		out.println("ERROR: " + name + " " + message);
		if (t!=null && traces) t.printStackTrace(out);
	}

	@Override
	public void fatal(Object message) {
		if (!isFatalEnabled()) return;
		out.println("FATAL: " + name + " " + message);
		if (message != null && message instanceof Throwable && traces)
			((Throwable)message).printStackTrace(out);
	}

	@Override
	public void fatal(Object message, Throwable t) {
		if (!isFatalEnabled()) return;
		out.println("FATAL: " + name + " " + message);
		if (t!=null && traces) t.printStackTrace(out);
	}

	@Override
	public Log getInstance(Class<?> clazz) {
		return getInstance(clazz.getName());
	}

	@Override
	public Log getInstance(String name) {
		return new ConsoleFactory(name,out);
	}

	@Override
	public void info(Object message) {
		if (!isInfoEnabled()) return;
		out.println("INFO : " + name + " " + message);
		if (message != null && message instanceof Throwable && traces)
			((Throwable)message).printStackTrace(out);
	}

	@Override
	public void info(Object message, Throwable t) {
		if (!isInfoEnabled()) return;
		out.println("INFO : " + name + " " + message);
		if (t!=null && traces) t.printStackTrace(out);
	}

	@Override
	public boolean isDebugEnabled() {
		return localTrace || level.ordinal() <= LEVEL.DEBUG.ordinal();
	}

	@Override
	public boolean isErrorEnabled() {
		return localTrace || level.ordinal() <= LEVEL.ERROR.ordinal();
	}

	@Override
	public boolean isFatalEnabled() {
		return localTrace || level.ordinal() <= LEVEL.FATAL.ordinal();
	}

	@Override
	public boolean isInfoEnabled() {
		return localTrace || level.ordinal() <= LEVEL.INFO.ordinal();
	}

	@Override
	public boolean isTraceEnabled() {
		return localTrace || level.ordinal() <= LEVEL.TRACE.ordinal();
	}

	@Override
	public boolean isWarnEnabled() {
		return localTrace || level.ordinal() <= LEVEL.WARN.ordinal();
	}

	@Override
	public void trace(Object message) {
		if (isTrace()) {
			out.println("TRACE: " + name + " " + message);
			if (message != null && message instanceof Throwable && traces)
				((Throwable)message).printStackTrace(out);
		}
	}

	@Override
	public void trace(Object message, Throwable t) {
		if (!isTrace()) return;
		out.println("TRACE: " + name + " " + message);
		if (t!=null && traces) t.printStackTrace(out);
	}

	@Override
	public void warn(Object message) {
		if (!isWarnEnabled()) return;
		out.println("WARN : " + name + " " + message);
		if (message != null && message instanceof Throwable && traces)
			((Throwable)message).printStackTrace(out);
	}

	@Override
	public void warn(Object message, Throwable t) {
		if (!isWarnEnabled()) return;
		out.println("WARN : " + name + " " + message);
		if (t!=null && traces) t.printStackTrace(out);
	}

}
