package de.mhus.lib.logging;

import java.util.Observable;
import java.util.Observer;

import de.mhus.lib.MSingleton;

/**
 * Got the interface from apache-commons-logging. I need to switch because its not working
 * in eclipse plugins correctly.
 * 
 * @author mikehummel
 *
 */
public abstract class Log implements Observer {

	public enum LEVEL {TRACE,DEBUG,INFO,WARN,ERROR,FATAL};
	/**
	 * Return the current log handler singleton.
	 * 
	 * @return
	 */
    public static Log getFactory() {
    	return MSingleton.instance().getLog();
    }

	protected boolean localTrace;
	protected String name;
    
    /**
     * Convenience method to derive a name from the specified class and
     * call <code>getInstance(String)</code> with it.
     *
     * @param clazz Class for which a suitable Log name will be derived
     *     */
    public abstract Log getInstance(Class<?> clazz);


    /**
     * <p>Construct (if necessary) and return a <code>Log</code> instance,
     * using the factory's current set of configuration attributes.</p>
     *
     * <p><strong>NOTE</strong> - Depending upon the implementation of
     * the <code>LogFactory</code> you are using, the <code>Log</code>
     * instance you are returned may or may not be local to the current
     * application, and may or may not be returned again on a subsequent
     * call with the same name argument.</p>
     *
     * @param name Logical name of the <code>Log</code> instance to be
     *  returned (the meaning of this name is only known to the underlying
     *  logging implementation that is being wrapped)
     */
    public abstract Log getInstance(String name);

    /**
     * Convenience method to return a named logger, without the application
     * having to care about factories.
     *
     * @param clazz Class from which a log name will be derived
     */
    public static Log getLog(Class<?> clazz) {

        Log log = (getFactory().getInstance(clazz));
        log.name = clazz.getCanonicalName();
        log.update(null, null);
        MSingleton.instance().registerConfigListener(log);
        return log;
    }

    /**
     * Convenience method to return a named logger, without the application
     * having to care about factories.
     *
     * @param name Logical name of the <code>Log</code> instance to be
     *  returned (the meaning of this name is only known to the underlying
     *  logging implementation that is being wrapped)
     */
    public static Log getLog(String name) {

        Log log = (getFactory().getInstance(name));
        log.name = name;
        log.update(null, null);
        MSingleton.instance().registerConfigListener(log);
        return log;
    }

    public void update(Observable o, Object arg) {
        setTrace(MSingleton.instance().getConfig().getBoolean(name + ".TRACE" , MSingleton.instance().getConfig().getBoolean("TRACE",false) ));
    }
    
    /**
     * <p> Is debug logging currently enabled? </p>
     *
     * <p> Call this method to prevent having to perform expensive operations
     * (for example, <code>String</code> concatenation)
     * when the log level is more than debug. </p>
     */
    public abstract boolean isDebugEnabled();


    /**
     * <p> Is error logging currently enabled? </p>
     *
     * <p> Call this method to prevent having to perform expensive operations
     * (for example, <code>String</code> concatenation)
     * when the log level is more than error. </p>
     */
    public abstract boolean isErrorEnabled();


    /**
     * <p> Is fatal logging currently enabled? </p>
     *
     * <p> Call this method to prevent having to perform expensive operations
     * (for example, <code>String</code> concatenation)
     * when the log level is more than fatal. </p>
     */
    public abstract boolean isFatalEnabled();


    /**
     * <p> Is info logging currently enabled? </p>
     *
     * <p> Call this method to prevent having to perform expensive operations
     * (for example, <code>String</code> concatenation)
     * when the log level is more than info. </p>
     */
    public abstract boolean isInfoEnabled();


    /**
     * <p> Is trace logging currently enabled? </p>
     *
     * <p> Call this method to prevent having to perform expensive operations
     * (for example, <code>String</code> concatenation)
     * when the log level is more than trace. </p>
     */
    public abstract boolean isTraceEnabled();


    /**
     * <p> Is warn logging currently enabled? </p>
     *
     * <p> Call this method to prevent having to perform expensive operations
     * (for example, <code>String</code> concatenation)
     * when the log level is more than warn. </p>
     */
    public abstract boolean isWarnEnabled();


    // -------------------------------------------------------- Logging Methods

    /**
     * Log a message in trace, it will automatically append the objects if trace is enabled. Can Also add a trace.
     * This is the local trace method. The trace will only written if the local trace is switched on.
     */
    public void t(Object ... msg) {
    	if (!localTrace) return;
    	StringBuffer sb = new StringBuffer();
    	Throwable error = null;
    	int cnt=0;
    	for (Object o : msg) {
			error = serialize(sb,o, error);
    		cnt++;
    	}
    	trace(sb.toString(),error);
    }

    private Throwable serialize(StringBuffer sb, Object o, Throwable error) {
    	if (o == null) {
			sb.append("[null]");
    	} else
		if (o instanceof Throwable) {
			if (error == null) return (Throwable)o;
			// another error
			sb.append("[").append(o).append("]");
		} else
    	if (o.getClass().isArray()) {
    		sb.append("{");
    		for (Object p : (Object[])o) {
    			error = serialize(sb, p, error);
    		}
    		sb.append("}");
    	} else
    		sb.append("[").append(o).append("]");
		return error;
	}


	/**
     * Log a message in trace, it will automatically append the objects if trace is enabled. Can Also add a trace.
     */
    public void tt(Object ... msg) {
    	if (!isTraceEnabled()) return;
    	StringBuffer sb = new StringBuffer();
    	Throwable error = null;
    	int cnt=0;
    	for (Object o : msg) {
			error = serialize(sb,o, error);
    		cnt++;
    	}
    	trace(sb.toString(),error);
    }

    /**
     * Log a message in debug, it will automatically append the objects if debug is enabled. Can Also add a trace.
     */
    public void d(Object ... msg) {
    	if (!isDebugEnabled()) return;
    	StringBuffer sb = new StringBuffer();
    	Throwable error = null;
    	int cnt=0;
    	for (Object o : msg) {
			error = serialize(sb,o, error);
    		cnt++;
    	}
    	debug(sb.toString(),error);
    }

    /**
     * Log a message in info, it will automatically append the objects if debug is enabled. Can Also add a trace.
     */
    public void i(Object ... msg) {
    	if (!isInfoEnabled()) return;
    	StringBuffer sb = new StringBuffer();
    	Throwable error = null;
    	int cnt=0;
    	for (Object o : msg) {
			error = serialize(sb,o, error);
    		cnt++;
    	}
    	info(sb.toString(),error);
    }

    /**
     * Log a message in warn, it will automatically append the objects if debug is enabled. Can Also add a trace.
     */
    public void w(Object ... msg) {
    	if (!isWarnEnabled()) return;
    	StringBuffer sb = new StringBuffer();
    	Throwable error = null;
    	int cnt=0;
    	for (Object o : msg) {
			error = serialize(sb,o, error);
    		cnt++;
    	}
    	warn(sb.toString(),error);
    }

    /**
     * Log a message in error, it will automatically append the objects if debug is enabled. Can Also add a trace.
     */
    public void e(Object ... msg) {
    	if (!isErrorEnabled()) return;
    	StringBuffer sb = new StringBuffer();
    	Throwable error = null;
    	int cnt=0;
    	for (Object o : msg) {
			error = serialize(sb,o, error);
   		cnt++;
    	}
    	error(sb.toString(),error);
    }

    /**
     * Log a message in info, it will automatically append the objects if debug is enabled. Can Also add a trace.
     */
    public void f(Object ... msg) {
    	StringBuffer sb = new StringBuffer();
    	Throwable error = null;
    	int cnt=0;
    	for (Object o : msg) {
			error = serialize(sb,o, error);
    		cnt++;
    	}
    	fatal(sb.toString(),error);
    }

    /**
     * <p> Log a message with trace log level. </p>
     *
     * @param message log this message
     */
    public abstract void trace(Object message);


    /**
     * <p> Log an error with trace log level. </p>
     *
     * @param message log this message
     * @param t log this cause
     */
    public abstract void trace(Object message, Throwable t);


    /**
     * <p> Log a message with debug log level. </p>
     *
     * @param message log this message
     */
    public abstract void debug(Object message);


    /**
     * <p> Log an error with debug log level. </p>
     *
     * @param message log this message
     * @param t log this cause
     */
    public abstract void debug(Object message, Throwable t);


    /**
     * <p> Log a message with info log level. </p>
     *
     * @param message log this message
     */
    public abstract void info(Object message);


    /**
     * <p> Log an error with info log level. </p>
     *
     * @param message log this message
     * @param t log this cause
     */
    public abstract void info(Object message, Throwable t);


    /**
     * <p> Log a message with warn log level. </p>
     *
     * @param message log this message
     */
    public abstract void warn(Object message);


    /**
     * <p> Log an error with warn log level. </p>
     *
     * @param message log this message
     * @param t log this cause
     */
    public abstract void warn(Object message, Throwable t);


    /**
     * <p> Log a message with error log level. </p>
     *
     * @param message log this message
     */
    public abstract void error(Object message);


    /**
     * <p> Log an error with error log level. </p>
     *
     * @param message log this message
     * @param t log this cause
     */
    public abstract void error(Object message, Throwable t);


    /**
     * <p> Log a message with fatal log level. </p>
     *
     * @param message log this message
     */
    public abstract void fatal(Object message);


    /**
     * <p> Log an error with fatal log level. </p>
     *
     * @param message log this message
     * @param t log this cause
     */
    public abstract void fatal(Object message, Throwable t);

	public void setTrace(boolean localTrace) {
		this.localTrace = localTrace;
	}

	public boolean isTrace() {
		return localTrace;
	}

	/**
	 * Use the name of the caller
	 * @return
	 */
	public static Log getLog() {
		StackTraceElement[] stack = Thread.currentThread().getStackTrace();
		// for (StackTraceElement e : stack) System.out.println(e.getClassName());
		return getLog(stack[2].getClassName());
	}


	public String getName() {
		return name;
	}

	public String toString() {
		return getName() + "@Log";
	}
	
}
