package de.mhus.lib.logging;

/*
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.Priority;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.xml.DOMConfigurator;

import com.mysql.jdbc.log.LogFactory;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.config.XmlConfig;

/**
 * <p>Concrete subclass of {@link LogFactory} specific to log4j.
 *
 * @author Costin Manolache
 */
public final class Log4JFactory extends Log {

    private static final boolean is12 = Priority.class.isAssignableFrom(Level.class);
    private static final String FQCN = Log4JFactory.class.getName();

    private String name;
	private Logger logger;


    public Log4JFactory(IConfig config) {
    	super();
    	if (config != null) {
    		
    		IConfig ccc = config.getConfig("configuration");
    		String configFile = config.getExtracted("configuration");
    		
    		if (ccc != null && ccc instanceof XmlConfig) {
    			MLog.t("configure inline");
    			DOMConfigurator.configure( ((XmlConfig)ccc).getXmlElement() );
    		} else
    		if (configFile == null) {
    			// NOOP
    		} else
    		if (configFile.endsWith(".properties")) {
    			MLog.t("configure properties",configFile);
    			PropertyConfigurator.configureAndWatch(configFile);
    		} else
    		if (configFile.endsWith(".xml")) {
    			MLog.t("configure xml",configFile);
    			DOMConfigurator.configureAndWatch(configFile);
    		}
    		
    	}
    }
    
	public Log4JFactory() {
        super();
    }

    /** For use with a log4j factory.
     */
    private Log4JFactory(Logger logger ) {
        this.name = logger.getName();
        this.logger=logger;
    }
    
    /**
     * Convenience method to derive a name from the specified class and
     * call <code>getInstance(String)</code> with it.
     *
     * @param clazz Class for which a suitable Log name will be derived
     *
     * @exception LogConfigurationException if a suitable <code>Log</code>
     *  instance cannot be returned
     */
    public Log getInstance(Class<?> clazz)
    {
        Log4JFactory instance = new Log4JFactory( Logger.getLogger( clazz ));
        return instance;
    }


    public Log getInstance(String name)
    {
        Log4JFactory instance = new Log4JFactory( Logger.getLogger( name ));
        return instance;
    }

    // ------------
    

    /**
     * Log a message to the Log4j Logger with <code>TRACE</code> priority.
     * Currently logs to <code>DEBUG</code> level in Log4J.
     */
    public void trace(Object message) {
    	if (!isTrace()) return;
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.DEBUG, message, null );
        } else {
            getLogger().log(FQCN, Level.DEBUG, message, null );
        }
    }


    /**
     * Log an error to the Log4j Logger with <code>TRACE</code> priority.
     * Currently logs to <code>DEBUG</code> level in Log4J.
     */
    public void trace(Object message, Throwable t) {
    	if (!isTrace()) return;
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.DEBUG, message, t );
        } else {
            getLogger().log(FQCN, Level.DEBUG, message, t );
        }
    }


    /**
     * Log a message to the Log4j Logger with <code>DEBUG</code> priority.
     */
    public void debug(Object message) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.DEBUG, message, null );
        } else {
            getLogger().log(FQCN, Level.DEBUG, message, null );
        }
    }

    /**
     * Log an error to the Log4j Logger with <code>DEBUG</code> priority.
     */
    public void debug(Object message, Throwable t) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.DEBUG, message, t );
        } else {
            getLogger().log(FQCN, Level.DEBUG, message, t );
        }
    }


    /**
     * Log a message to the Log4j Logger with <code>INFO</code> priority.
     */
    public void info(Object message) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.INFO, message, null );
        } else {
            getLogger().log(FQCN, Level.INFO, message, null );
        }    
    }


    /**
     * Log an error to the Log4j Logger with <code>INFO</code> priority.
     */
    public void info(Object message, Throwable t) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.INFO, message, t );
        } else {
            getLogger().log(FQCN, Level.INFO, message, t );
        }
    }


    /**
     * Log a message to the Log4j Logger with <code>WARN</code> priority.
     */
    public void warn(Object message) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.WARN, message, null );
        } else {
            getLogger().log(FQCN, Level.WARN, message, null );
        }
    }


    /**
     * Log an error to the Log4j Logger with <code>WARN</code> priority.
     */
    public void warn(Object message, Throwable t) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.WARN, message, t );
        } else {
            getLogger().log(FQCN, Level.WARN, message, t );
        }
    }


    /**
     * Log a message to the Log4j Logger with <code>ERROR</code> priority.
     */
    public void error(Object message) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.ERROR, message, null );
        } else {
            getLogger().log(FQCN, Level.ERROR, message, null );
        }
    }


    /**
     * Log an error to the Log4j Logger with <code>ERROR</code> priority.
     */
    public void error(Object message, Throwable t) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.ERROR, message, t );
        } else {
            getLogger().log(FQCN, Level.ERROR, message, t );
        }
    }


    /**
     * Log a message to the Log4j Logger with <code>FATAL</code> priority.
     */
    public void fatal(Object message) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.FATAL, message, null );
        } else {
            getLogger().log(FQCN, Level.FATAL, message, null );
        }
    }


    /**
     * Log an error to the Log4j Logger with <code>FATAL</code> priority.
     */
    public void fatal(Object message, Throwable t) {
        if(is12) {
            getLogger().log(FQCN, (Priority) Level.FATAL, message, t );
        } else {
            getLogger().log(FQCN, Level.FATAL, message, t );
        }
    }


    /**
     * Return the native Logger instance we are using.
     */
    public Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger(name);
        }
        return (this.logger);
    }


    /**
     * Check whether the Log4j Logger used is enabled for <code>DEBUG</code> priority.
     */
    public boolean isDebugEnabled() {
        return getLogger().isDebugEnabled();
    }


     /**
     * Check whether the Log4j Logger used is enabled for <code>ERROR</code> priority.
     */
    public boolean isErrorEnabled() {
        if(is12) {
            return getLogger().isEnabledFor((Priority) Level.ERROR);
        } else {
            return getLogger().isEnabledFor(Level.ERROR);
        }
    }


    /**
     * Check whether the Log4j Logger used is enabled for <code>FATAL</code> priority.
     */
    public boolean isFatalEnabled() {
        if(is12) {
            return getLogger().isEnabledFor((Priority) Level.FATAL);
        } else {
            return getLogger().isEnabledFor(Level.FATAL);
        }
    }


    /**
     * Check whether the Log4j Logger used is enabled for <code>INFO</code> priority.
     */
    public boolean isInfoEnabled() {
        return getLogger().isInfoEnabled();
    }


    /**
     * Check whether the Log4j Logger used is enabled for <code>TRACE</code> priority.
     * For Log4J, this returns the value of <code>isDebugEnabled()</code>
     */
    public boolean isTraceEnabled() {
        return getLogger().isDebugEnabled();
    }

    /**
     * Check whether the Log4j Logger used is enabled for <code>WARN</code> priority.
     */
    public boolean isWarnEnabled() {
        if(is12) {
            return getLogger().isEnabledFor((Priority) Level.WARN);
        } else {
            return getLogger().isEnabledFor(Level.WARN);
        }
    }

}
