package de.mhus.lib.logging;

import java.util.LinkedList;

import de.mhus.lib.config.IConfig;

/**
 * Distribute the log to a bundle of other loggers. The first logger is the main logger how
 * is responsible for the log level.
 * 
 * @author mikehummel
 *
 */
public class YLogger extends Log {
	
	private Log[] targets;

	public YLogger(IConfig config) throws Exception {
		LinkedList<Log> list = new LinkedList<Log>();
		for (IConfig c : config.getConfigBundle("logger")) {
			Log next = LogInitializer.createLogFactroy(c);
			if (next != null) list.add(next);
		}
		targets = list.toArray(new Log[list.size()]);
	}
	
	public YLogger(Log ... targets ) {
		this.targets = targets;
	}
	
	public YLogger(Log[] targets2, String name) {
		targets = new Log[targets2.length];
		for (int i = 0; i < targets.length; i++) {
			targets[i] = targets2[i].getInstance(name);
			targets[i].name = name;
			targets[i].update(null, null);
		}
	}

	public YLogger(Log[] targets2, Class<?> clazz) {
		targets = new Log[targets2.length];
		for (int i = 0; i < targets.length; i++) {
			targets[i] = targets2[i].getInstance(clazz);
			targets[i].name = clazz.getCanonicalName();
			targets[i].update(null, null);
		}
	}

	@Override
	public void warn(Object message, Throwable t) {
		for ( Log log2 : targets)
			log2.warn(message, t);
	}
	
	@Override
	public void warn(Object message) {
		for ( Log log2 : targets)
			log2.warn(message);
	}
	
	@Override
	public void trace(Object message, Throwable t) {
		for ( Log log2 : targets)
			log2.trace(message,t);
	}
	
	@Override
	public void trace(Object message) {
		for ( Log log2 : targets)
			log2.trace(message);
	}
	
	@Override
	public boolean isWarnEnabled() {
		for ( Log log2 : targets)
			if (log2.isWarnEnabled()) return true;
		return false;
	}
	
	@Override
	public boolean isTraceEnabled() {
		for ( Log log2 : targets)
			if (log2.isTraceEnabled()) return true;
		return false;
	}
	
	@Override
	public boolean isInfoEnabled() {
		for ( Log log2 : targets)
			if (log2.isInfoEnabled()) return true;
		return false;
	}
	
	@Override
	public boolean isFatalEnabled() {
		for ( Log log2 : targets)
			if (log2.isFatalEnabled()) return true;
		return false;
	}
	
	@Override
	public boolean isErrorEnabled() {
		for ( Log log2 : targets)
			if (log2.isErrorEnabled()) return true;
		return false;
	}
	
	@Override
	public boolean isDebugEnabled() {
		for ( Log log2 : targets)
			if (log2.isDebugEnabled()) return true;
		return false;
	}
	
	@Override
	public void info(Object message, Throwable t) {
		for ( Log log2 : targets)
			log2.info(message, t);
	}
	
	@Override
	public void info(Object message) {
		for ( Log log2 : targets)
			log2.info(message);
	}
	
	@Override
	public Log getInstance(String name) {
		// return targets[0].getInstance(name);
		return new YLogger(targets,name);
	}
	
	@Override
	public Log getInstance(Class<?> clazz) {
		return new YLogger(targets,clazz);
	}
	
	@Override
	public void fatal(Object message, Throwable t) {
		for ( Log log2 : targets)
			log2.fatal(message, t);
	}
	
	@Override
	public void fatal(Object message) {
		for ( Log log2 : targets)
			log2.fatal(message);
	}
	
	@Override
	public void error(Object message, Throwable t) {
		for ( Log log2 : targets)
			log2.error(message, t);
	}
	
	@Override
	public void error(Object message) {
		for ( Log log2 : targets)
			log2.error(message);
	}
	
	@Override
	public void debug(Object message, Throwable t) {
		for ( Log log2 : targets)
			log2.debug(message, t);
	}
	
	@Override
	public void debug(Object message) {
		for ( Log log2 : targets)
			log2.debug(message);
	}

	@Override
	public void setTrace(boolean localTrace) {
		super.setTrace(localTrace);
		for ( Log log2 : targets)
			log2.setTrace(localTrace);
	}

	
}
