package de.mhus.lib.persistence;

import java.io.File;
import java.util.Date;
import java.util.LinkedList;

import de.mhus.lib.MCast;
import de.mhus.lib.MFile;
import de.mhus.lib.MSingleton;
import de.mhus.lib.MString;
import de.mhus.lib.MSystem;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.jmx.JmxManaged;

public class DefaultPersistence extends MPersistenceManager {

	private DefaultScope global;
	private DefaultScope session;

	@Override
	public void flush() {
		global.flush();
		session.flush();
	}

	@Override
	public void init(IConfig config) {
		String path = config.getExtracted("path");
		File globalDir = (path == null ? new File(MSingleton.instance().getApplicationDirectory(), "persistence") : new File(path) );
		globalDir.mkdirs();
		if (!globalDir.exists() || !globalDir.canWrite()){
			log().w("Persistence not possible",globalDir);
			return;
		}
		
		File sessionDir = new File(globalDir, "session_" + MFile.normalize(MCast.toIsoDateTime(new Date())) + "_" + MSystem.getPid());
		sessionDir.mkdirs();
		if (!sessionDir.exists() || !sessionDir.canWrite()){
			log().w("Persistence not possible",sessionDir);
			return;
		}
		
		global = new DefaultScope(globalDir);
		session = new DefaultScope(sessionDir);
		
		setActive(true);
	}

	@Override
	public void close() {
		setActive(false);
		global.close();
		session.close();
	}

	@Override
	public Scope sessionScope() {
		return session;
	}

	@Override
	public Scope globalScope() {
		return global;
	}

	@Override
	public Scope getSession(String sessionId) {
		sessionId = MFile.normalize(sessionId);
		File sessionDir = new File(global.getFile(), "session_" + sessionId);	
		if (!sessionDir.exists() || !sessionDir.isDirectory()) return null;
		return new DefaultScope(sessionDir);
	}

	@Override
	public String[] getSessionIds() {
		LinkedList<String> out = new LinkedList<String>();
		for (File f : global.getFile().listFiles()) {
			if (f.isDirectory() && f.getName().startsWith("session_"))
				out.add(MString.afterIndex(f.getName(),'_'));
		}
		return out.toArray(new String[out.size()]);
	}

}
