package de.mhus.lib.persistence;

import java.io.File;
import java.util.LinkedList;
import java.util.WeakHashMap;

import org.w3c.dom.Element;


import de.mhus.lib.MException;
import de.mhus.lib.MFile;
import de.mhus.lib.MString;
import de.mhus.lib.MXml;
import de.mhus.lib.config.ConfigUtil;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.config.XmlConfigFile;
import de.mhus.lib.jmx.JmxManaged;

public class DefaultScope extends Scope {

	private File dir;
	private WeakHashMap<String, XmlConfigFile> cache = new WeakHashMap<String, XmlConfigFile>();

	public DefaultScope(File dir) {
		this.dir = dir;
		log().t(getSessionId(),"directory",dir);
	}

	public void close() {
		flush();
	}

	public void flush() {
		synchronized (this) {
			for (XmlConfigFile config : cache.values()) {
				try {
					config.save();
				} catch (MException e) {
					log().w(config,e);
				}
			}
		}
	}

	@Override
	public IConfig getPersistence(String name) {
		synchronized (this) {
			XmlConfigFile config = cache.get(name);
			if (config == null) {
				try {
					config = new XmlConfigFile(new File(dir,"config_" + MFile.normalize(name) + ".xml"));
					cache.put(name,config);
				} catch (Throwable t) {
					log().w(dir,name,t);
				}
			}
			return config;
		}
	}

	@Override
	public File getPersistenceFile(String name) {
		return new File(dir,"file_" + MFile.normalize(name));
	}

	public File getFile() {
		return dir;
	}

	@Override
	public boolean isPersistenceFile() {
		return true;
	}

	@Override
	public String[] getPersistenceNames() {
		LinkedList<String> out = new LinkedList<String>();
		for (File f : dir.listFiles()) {
			if (f.isFile() && f.getName().startsWith("config_"))
				out.add(MString.beforeLastIndex(MString.afterIndex(f.getName(),'_'),'.'));
		}
		return out.toArray(new String[out.size()]);
	}

	@Override
	public String[] getPersistenceFileNames() {
		LinkedList<String> out = new LinkedList<String>();
		for (File f : dir.listFiles()) {
			if (f.isFile() && f.getName().startsWith("file_"))
				out.add(MString.afterIndex(f.getName(),'_'));
		}
		return out.toArray(new String[out.size()]);
	}

	@Override
	public String getSessionId() {
		if (dir.getName().startsWith("session_")) {
			return MString.afterIndex(dir.getName(),'_');
		}
		return "global";
	}

	@JmxManaged
	public String getPersistentFilePath(String name) {
		 File f = getPersistenceFile(name);
		 return f.exists() ? f.getAbsolutePath() : null;
	}
	
	@JmxManaged
	public String dumpPersistentConfig(String name) {
		IConfig p = getPersistence(name);
		if (p == null) return null;
		
		Element e = ((XmlConfigFile)p).getXmlElement();
		
		try {
			return "<pre>" + MXml.encode(MXml.toString(e, true)).replaceAll("&quot; ", "&quot; \n") + "</pre>";
		} catch (Exception e1) {
			return null;
		}
	}
	
}
