package de.mhus.lib.persistence;

import java.util.Hashtable;

import de.mhus.lib.config.IConfig;
import de.mhus.lib.jmx.JmxManaged;
import de.mhus.lib.jmx.MJmx;

@JmxManaged(descrition = "Application Persistence")
public abstract class MPersistenceManager extends MJmx {

	private boolean active = false;
	private Hashtable<String,Scope> sessionJmxCache = new Hashtable<String, Scope>();
	
	public abstract void close();
	
	@JmxManaged(descrition="Flush the changed data into the data store if possible")
	public abstract void flush();
	
	public abstract void init(IConfig config) throws Exception;

	public abstract Scope sessionScope();
	
	public abstract Scope globalScope();

	/**
	 * Return the session scope object or null if not found.
	 * 
	 * @param sessionId
	 * @return
	 */
	@JmxManaged
	public abstract Scope getSession(String sessionId);

	@JmxManaged
	public abstract String[] getSessionIds();
	
	protected void setActive(boolean active) {
		this.active = active;
	}

	@JmxManaged
	public boolean isActive() {
		return active;
	}
	
	/**
	 * this will load the session in a local cache for jms reasons
	 * @param sessionId
	 * @return
	 */
	@JmxManaged
	public boolean loadSession(String sessionId) {
		Scope session = getSession(sessionId);
		if (session != null) {
			sessionJmxCache.put(sessionId,session);
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * this will unload the session from a local cache for jms reasons
	 * @param sessionId
	 */
	@JmxManaged
	public void unloadSession(String sessionId) {
		sessionJmxCache.remove(sessionId);
	}
	
}
