package de.mhus.lib.persistence;

import java.io.File;
import java.io.IOException;

import org.rrd4j.ConsolFun;
import org.rrd4j.DsType;
import org.rrd4j.core.RrdBackendFactory;
import org.rrd4j.core.RrdDb;
import org.rrd4j.core.RrdDef;
import org.rrd4j.core.Sample;

import de.mhus.lib.MFile;
import de.mhus.lib.MSystem;
import de.mhus.lib.lang.MObject;

public class RoundRobin extends MObject {

	private RrdDb rrdDb;
	private Sample sample;
	private long lastTime;

	/**
	 * 
	 * @param heartbeat
	 * @param path
	 * @throws IOException
	 */
	public RoundRobin(long heartbeat, File path) throws IOException {
		if (path == null ) path = MFile.createTempFile(MSystem.getPid(), "rrd");
		RrdDef rrdDef = new RrdDef(path.getAbsolutePath());
		rrdDef.setStartTime(System.currentTimeMillis());
		rrdDef.addDatasource("speed", DsType.COUNTER, heartbeat, Double.NaN, Double.NaN);
		rrdDef.addArchive(ConsolFun.AVERAGE, 0.5, 1, 24);
		rrdDef.addArchive(ConsolFun.AVERAGE, 0.5, 6, 10);
		// RrdBackendFactory factory = RrdBackendFactory.getFactory("MEMORY");
		rrdDb = new RrdDb(rrdDef);
		sample = rrdDb.createSample();
	}
	
	public void add(double value) {
		try {
			long time = System.currentTimeMillis();
			if (time / 1000 == lastTime) return; // will loose one secound ...
			lastTime = time / 1000;
			sample.setTime(time);
			sample.setValues(value);
			sample.update();
		} catch (IOException e) {
			log().t(e);
		}
	}
	
	public String dump() {
		return sample.dump();
	}
	/*
	private class MyFactory extends RrdMemoryBackendFactory {
		
		HashSet<String> knownIds = new HashSet<String>();
		
	    protected RrdBackend open(String id, boolean readOnly) throws IOException {
	    	knownIds.add(id);
	    	return super.open(id, readOnly);
	    }
	    
	    public boolean delete(String id) {
	    	knownIds.remove(id);
	    	return super.delete(id);
	    }
	    
	}
	*/
}
