package de.mhus.lib.sql;

import de.mhus.lib.MException;
import de.mhus.lib.MSingleton;
import de.mhus.lib.logging.Log;
import de.mhus.lib.parser.Parser;

/**
 * The class capsulate the real connection to bring it back into the pool if the connection in no more needed - closed
 * or cleanup by the gc.
 * 
 * @author mikehummel
 *
 */
public class DbConnectionProxy implements DbConnection {

	private static Log log = Log.getLog(DbConnectionProxy.class);
	private DbConnection instance;
	private long id = MSingleton.instance().nextUniqueId();
	private StackTraceElement[] createStackTrace;

	public DbConnectionProxy(DbConnection instance) {
		if (log.isTrace()) {
			createStackTrace = Thread.currentThread().getStackTrace();
			instance.setUsedTrace(createStackTrace);
		}
		this.instance = instance;
		log.t(id,"created",instance.getInstanceId());
	}
	
	public void commit() throws Exception {
		
		instance.commit();
	}

	public boolean isReadOnly() throws Exception {
		return instance.isReadOnly();
	}

	public void rollback() throws Exception {
		instance.rollback();
	}

	public DbStatement getStatement(String name) throws MException {
		return instance.getStatement(name);
	}

	public boolean isClosed() {
		if (instance == null) return true;
		return instance.isClosed();
	}

	public boolean isUsed() {
		return instance.isUsed();
	}

	public void setUsed(boolean used) {
		if (instance == null) return;
		instance.setUsed(used);
		if (!used) instance  = null; // invalidate this proxy
	}

	public void close() {
		if (instance == null) return;
		log.t(id,"close",instance.getInstanceId());
		setUsed(false); // close of the proxy will free the connection
	}
	
	@Override
	protected void finalize() throws Throwable {
		log.t(id,"finalized",instance.getInstanceId());
		if (instance != null) {
			log.d(id,"final closed",instance.getInstanceId(),createStackTrace);
			setUsed(false);
		}
		super.finalize();
	}

	@Override
	public DbStatement createStatement(String sql, String language) throws MException {
		return instance.createStatement(sql,language);
	}

	@Override
	public long getInstanceId() {
		return id;
	}

	@Override
	public Parser createQueryCompiler(String language) throws MException {
		return instance.createQueryCompiler(language);
	}

	@Override
	public DbConnection instance() {
		return instance;
	}

	@Override
	public DbStatement createStatement(DbPrepared dbPrepared) {
		return instance.createStatement(dbPrepared);
	}

	@Override
	public void setUsedTrace(StackTraceElement[] createStackTrace) {
		
	}

	@Override
	public StackTraceElement[] getUsedTrace() {
		return createStackTrace;
	}

	@Override
	public String getDefaultLanguage() {
		return instance.getDefaultLanguage();
	}

	@Override
	public String[] getLanguages() {
		return instance.getLanguages();
	}

	@Override
	public DbStatement createStatement(String sql) throws MException {
		return instance.createStatement(sql);
	}
	
}
