/*
 *  Copyright (C) 2002-2004 Mike Hummel
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package de.mhus.lib.sql;

import de.mhus.lib.MActivator;
import de.mhus.lib.config.IConfig;
import de.mhus.lib.lang.MObject;

/**
 * Provide the database connections and other db specific stuff.
 * The default implementation is the JdbcProvider. Implement this to
 * create a completely new kind of database for the framework.
 * 
 * @author mikehummel
 *
 */
public abstract class DbProvider extends MObject {
	
//	private static Log log = Log.getLog(DbProvider.class);

	protected IConfig config;
	protected MActivator activator;

	/**
	 * Returns a new DbConnection for this kind of database.
	 * 
	 * @return
	 * @throws Exception
	 */
	public abstract InternalDbConnection createConnection() throws Exception;

	/**
	 * Set configuration element and activator.
	 * 
	 * @param config
	 * @param activator
	 */
	public void init(IConfig config,MActivator activator) {
		this.config = config;
		this.activator = activator;
	}
	
	/**
	 * Returns the predefined statement by this name.
	 * TODO need to manipulate the set of statements from outside.
	 * 
	 * @param name
	 * @return The query string or null.
	 */
	public String[] getQuery(String name) {		
		IConfig query = config.getConfig("queries");
		String queryLanguage = null;
		String queryString = query.getString(name,null);
		String[] out = new String[] { queryLanguage, queryString };
		
		if (queryString == null) {
			for (IConfig q : query.getConfigBundle("query") ) {
				if (q.getString("name", "").equals(name)) {
					queryLanguage = q.getExtracted("language");
					queryString = q.getExtracted("string");
					out = new String[] { queryLanguage, queryString };
					break;
				}
			}
		}
		return out;
	}
	
	/**
	 * Returns the Dialect object for this database. It contains all deep specific
	 * abstraction functions to handle the database.
	 * 
	 * @return
	 */
	public abstract Dialect getDialect();

	/**
	 * Nice name of the connection - from configuration to identify it.
	 * 
	 * @return
	 */
	public String getName() {
		return config.getExtracted("name");
	}

	/**
	 * Returns the used activator.
	 * 
	 * @return
	 */
	public MActivator getActivator() {
		return activator;
	}
	
}
