package de.mhus.lib.sql;

import java.sql.Statement;
import java.text.SimpleDateFormat;

import de.mhus.lib.MDate;
import de.mhus.lib.config.IConfig;

/**
 * This class can compare a configuration with a database table structure
 * and can modify the database structure without deleting existing tables.
 * 
 * TODO: on request: remove other columns
 * TODO: views, foreign keys
 * TODO: data !!!
 * @author mikehummel
 *
 */
public class DialectHsqldb extends DialectDefault {
	
	private static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	protected String getFieldConfig(IConfig f) {
		String type = getDbType(f);
		
		String ret = f.getString("name",null).toUpperCase() + " " + type;
		
		String def = f.getExtracted("default");
		if (def != null) {
			def = getDbDef(def);
			ret = ret + " DEFAULT " + def;
		}
		boolean notNull = f.getBoolean("notnull", false);
		if (notNull)
			ret = ret + " NOT NULL";
		else
			ret = ret + " NULL";			
		return ret;
	}

	protected String getFieldConfigWithoutExtras(IConfig f) {
		String type = getDbType(f);
		
		String ret = f.getString("name",null).toUpperCase() + " " + type;
		
//		String def = f.getExtracted("default");
//		if (def != null) {
//			def = getDbDef(def);
//			ret = ret + " DEFAULT " + def;
//		}
//		boolean notNull = f.getBoolean("notnull", false);
//		if (notNull)
//			ret = ret + " NOT NULL";
//		else
//			ret = ret + " NULL";			
		return ret;
	}

	
	@Override
	public String toSqlDateValue(MDate date) {
		synchronized (dateFormat) {
			return "TO_DATE('" + dateFormat.format(date.toDate()	) + "','YYYY-MM-DD HH:MI:SS')";
		}
	}
	
	@Override
	public String normalizeColumnName(String columnName) {
		return columnName.toUpperCase();
	}
	
	protected void alterColumn(Statement sth,String tn, IConfig cfield) {
		String sql = "ALTER TABLE " + tn + " ALTER COLUMN " + getFieldConfigWithoutExtras(cfield);
		log.t("alter table",sql);
		try {
			sth.execute(sql);
		} catch (Exception e) {
			log.i(sql,e);
		}

	}

	protected void alterTableChangePrimaryKey(Statement sth, String tn,
			String keys) {
		alterTableDropPrimaryKey(sth, tn);
		alterTableAddPrimaryKey(sth, tn, keys);
	}

	protected void recreateIndex(Statement sth, boolean unique, boolean btree,
			String iName, String table, String columns) {

		iName = table + iName;
		String sql = "DROP INDEX " + iName;
		log.t(sql);
		try {
			sth.execute(sql.toString());
		} catch (Exception e) {
			log.i(sql,e);
		}
		sql = "CREATE " + (unique ? "UNIQUE" : "" ) + " INDEX " + iName + (btree ? " USING BTREE" : "") +" ON "+table+ "("+columns+")";
		log.t(sql);
		try {
			sth.execute(sql.toString());
		} catch (Exception e) {
			log.i(sql,e);
		}
	}

	protected void createIndex(Statement sth, boolean unique, boolean btree,
			String iName, String table, String columns) {
		
		iName = table + iName;
		
		String sql = "CREATE " + (unique ? "UNIQUE" : "" ) + " INDEX " + iName + (btree ? " USING BTREE" : "") +" ON "+table+ "("+columns+")";
		log.t(sql);
		try {
			sth.execute(sql.toString());
		} catch (Exception e) {
			log.i(sql,e);
		}

	}
	
	protected boolean equalsIndexName(String table, String iName, String iName2) {
		return iName2.equals(table + iName);
	}

	public String normalizeTableName(String tableName) throws Exception {
		return tableName.toUpperCase() + "_";
	}

	public String normalizeIndexName(String tableName) throws Exception {
		return tableName.toUpperCase();
	}

}
