package de.mhus.lib.util;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

public class FallbackMap<K, V> implements Map<K, V> {

	private Map<K, V> map;
	private Map<K, V> fall;
	private boolean fro = true;
	
	public FallbackMap(Map<K, V> map, Map<K, V> fallback, boolean fallbackReadOnly) {
		this.map = map;
		fall = fallback;
		fro = fallbackReadOnly;
	}
	
	@Override
	public int size() {
		return Math.max(map.size(), fall.size());
	}

	@Override
	public boolean isEmpty() {
		return map.isEmpty() && fall.isEmpty();
	}

	@Override
	public boolean containsKey(Object key) {
		return map.containsKey(key) || fall.containsKey(key);
	}

	@Override
	public boolean containsValue(Object value) {
		return map.containsValue(value) || fall.containsValue(value);
	}

	@Override
	public V get(Object key) {
		V ret = map.get(key);
		if (ret == null) return fall.get(key);
		return ret;
	}

	@Override
	public V put(K key, V value) {
		V ret = get(key);
		map.put(key, value);
		return ret;
	}

	@Override
	public V remove(Object key) {
		V ret = get(key);
		map.remove(key);
		if (!fro) fall.remove(key);
		return ret;
	}

	@Override
	public void putAll(Map<? extends K, ? extends V> m) {
		map.putAll(m);
	}

	@Override
	public void clear() {
		map.clear();
		if (!fro) fall.clear();
	}

	@Override
	public Set<K> keySet() {
		return map.keySet();
	}

	@Override
	public Collection<V> values() {
		return map.values();
	}

	@Override
	public Set<java.util.Map.Entry<K, V>> entrySet() {
		return map.entrySet();
	}

}
